define(
    [
        'underscore',
        'jquery',
        'backbone',
        'controllers/BaseSearch',
        'models/search/Report',
        'views/report/Master',
        'views/shared/documentcontrols/dialogs/permissions_dialog/Master',
        'views/shared/reportcontrols/dialogs/schedule_dialog/Master',
        'views/shared/reportcontrols/dialogs/AccelerationDialog',
        'views/shared/reportcontrols/dialogs/embed_dialog/Master',
        'uri/route',
        'splunk.util'
    ],
    function(
        _,
        $,
        Backbone,
        BaseSearchController,
        ReportModel,
        ReportView,
        PermissionsDialog,
        ScheduleDialog,
        AccelerationDialog,
        EmbedDialog,
        route,
        splunkUtils
    ){
        return BaseSearchController.extend({
            initializeViewsAndEvents: function(){
                this.model.eventsViewerReport = new ReportModel();
                this.model.eventsViewerReport.setFromSplunkD(this.model.report.toSplunkD());
                //views
                this.children.reportView = new ReportView({
                    model: {
                        result: this.model.result,
                        summary: this.model.summary,
                        timeline: this.model.timeline,
                        searchJob: this.model.searchJob,
                        searchResultsColumn: this.model.resultJsonCols,
                        searchResultsRow: this.model.resultJsonRows,
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        eventsViewerReport: this.model.eventsViewerReport,
                        application: this.model.application,
                        timeRange: this.model.timeRange,
                        appLocal: this.model.appLocal,
                        user: this.model.user
                    },
                    collection: {
                        times: this.collection.times,
                        selectedFields: this.collection.selectedFields,
                        eventRenderers: this.collection.eventRenderers,
                        roles: this.collection.roles,
                        workflowActions: this.collection.workflowActions
                    }
                });

                //Events for the report model
                this.model.eventsViewerReport.entry.content.on('change:display.events.table.sortColumn change:display.events.table.sortDirection',_.debounce(function() {
                    this.model.report.entry.content.set({
                        'display.events.table.sortDirection': this.model.eventsViewerReport.entry.content.get('display.events.table.sortDirection'),
                        'display.events.table.sortColumn': this.model.eventsViewerReport.entry.content.get('display.events.table.sortColumn')
                    });
                }, 0), this);

                this.model.report.entry.content.on('change:display.prefs.events.offset change:display.events.maxLines change:display.events.raw.drilldown change:display.events.list.drilldown change:display.events.fields change:display.prefs.events.count change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.events.table.sortColumn change:display.events.table.sortDirection', _.debounce(function() {
                    this.fetchResult();
                }, 0), this);

                this.model.report.entry.content.on('change:display.event.fields', this.populateSelectedFieldsFromReport, this);
                this.collection.selectedFields.on('add remove reset', this.populateReportFromSelectedFields, this);

                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.events.fieldsFilter change:display.events.fieldCoverage', _.debounce(function() {
                    this.fetchSummary();
                }, 0), this);

                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime', _.debounce(function() {
                    this.fetchTimeline();
                }, 0), this);

                this.model.report.entry.content.on('change:display.prefs.events.count', function(){
                    if (this.model.report.entry.content.get('display.prefs.events.count')){
                        this.model.report.entry.content.set({'display.prefs.events.offset': '0'});
                    }
                }, this);

                this.model.report.entry.content.on('change:display.prefs.statistics.count', function(){
                    if (this.model.report.entry.content.get('display.prefs.statistics.count')){
                        this.model.report.entry.content.set({'display.prefs.statistics.offset': '0'});
                    }
                }, this);

                this.model.report.entry.content.on('change:display.general.enablePreview', function(){
                    if (splunkUtils.normalizeBoolean(this.model.report.entry.content.get('display.general.enablePreview'))) {
                        this.model.searchJob.enablePreview();
                    } else {
                        this.model.searchJob.disablePreview();
                    }
                }, this);

                this.model.report.entry.content.on('change:display.statistics.sortColumn change:display.statistics.sortDirection change:display.prefs.statistics.count change:display.prefs.statistics.offset', _.debounce(function() {
                    this.fetchResultJSONRows();
                }, 0), this);

                //arbitrators to determine when to push on the history stack
                this.model.report.entry.content.on('change', function(){
                    var changed = this.model.report.entry.content.changedAttributes(),
                        jobPopulationDeferred = $.Deferred(),
                        uiPrefsPopulationDeferred = $.Deferred();

                    //we have to delete the change to search so we don't override the search of the report
                    delete changed.search;
                    
                    this.populateJob(jobPopulationDeferred, changed);
                    this.populateUIPrefs(uiPrefsPopulationDeferred, changed);
                    
                    $.when(jobPopulationDeferred, uiPrefsPopulationDeferred).always(function() {
                        this.populateClassicUrl(changed);
                    }.bind(this));
                }, this);

                //listeners
                this.model.reportPristine.on('change:id', function(){
                    this.classicUrl.save({ s:this.model.reportPristine.id }, { replaceState: true });
                    var payload = this.model.reportPristine.toSplunkD();
                    payload.entry[0].content = {};
                    this.model.report.setFromSplunkD(payload);
                    this.model.eventsViewerReport.setFromSplunkD(payload);
                }, this);

                //Events on the searchJob model
                this.model.searchJob.on("prepared", function(){
                    this.registerSearchJobFriends();
                    this.model.report.setDisplayType(this.model.searchJob.isReportSearch());
                }, this);
                
                this.model.searchJob.on('destroy', function() {
                    this.classicUrl.save(
                        {
                            sid: "",
                            "display.prefs.events.offset": "",
                            "display.prefs.statistics.offset": "",
                            "display.statistics.sortColumn": "",
                            "display.statistics.sortDirection": "",
                            "display.events.table.sortColumn": "",
                            "display.events.table.sortDirection": "",
                            auto_pause: ""
                        },
                        {
                            replaceState: true
                        }
                    );
                }, this);

                this.model.searchJob.on("close", this.close, this);

                this.model.searchJob.on('reload', function() {
                    this.classicUrl.save({sid: undefined}, {trigger: true});
                }, this);
            },
            setReplacementDOM: function() {
                this.replacementDOM = this.children.reportView.render().el;

                switch(this.classicUrl.get('dialog')) {
                    case 'permissions':
                        this.showPermissionsDialog();
                        break;
                    case 'schedule':
                        this.showScheduleDialog();
                        break;
                    case 'acceleration':
                        this.showAccelerationDialog();
                        break;
                    case 'embed':
                        this.showEmbedDialog();
                        break;
                }
            },
            registerSearchJobFriends: function() {
                if(this.model.report.entry.content.get('display.visualizations.type') === 'mapping') {
                    // TODO [sff] should set the count from a report attribute here
                    this.model.resultJsonCols.fetchData.set({ show_metadata: false }, { silent: true });
                }
                BaseSearchController.prototype.registerSearchJobFriends.apply(this, arguments);
            },
            fetchResultJSONColumn: function() {
                if (this.model.resultJsonCols.id && this.model.report.entry.content.get('display.visualizations.show') === "1") {
                    if(this.model.report.entry.content.get('display.visualizations.type') === 'mapping') {
                        if(this.model.resultJsonCols.fetchData.get('search')) {
                            this.model.resultJsonCols.safeFetch();
                        }
                    }
                    else {
                        BaseSearchController.prototype.fetchResultJSONColumn.apply(this);
                    }
                }
            },
            showPermissionsDialog: function() {
                this.permissionsDialog = new PermissionsDialog({
                    model: {
                        document: this.model.reportPristine,
                        nameModel: this.model.reportPristine.entry,
                        user: this.model.user
                    },
                    collection: this.collection.roles,
                    onHiddenRemove: true,
                    nameLabel: _('Report').t()
                });

                $("body").append(this.permissionsDialog.render().el);
                this.permissionsDialog.show();

                this.classicUrl.save({ dialog: undefined }, { replaceState: true });

            },
            showScheduleDialog: function() {
                this.scheduleDialog = new ScheduleDialog({
                    model: {
                        report: this.model.reportPristine,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user
                    },
                    onHiddenRemove: true
                });

                $("body").append(this.scheduleDialog.render().el);
                this.scheduleDialog.show();

                this.classicUrl.save({ dialog: undefined }, { replaceState: true });

            },
            showAccelerationDialog: function() {
                this.accelerationDialog = new AccelerationDialog({
                    model: {
                        report: this.model.reportPristine,
                        application: this.model.application
                    },
                    onHiddenRemove: true
                });

                $("body").append(this.accelerationDialog.render().el);
                this.accelerationDialog.show();

                this.classicUrl.save({ dialog: undefined }, { replaceState: true });

            },
            showEmbedDialog: function() {
                this.embedDialog = new EmbedDialog({
                    model: {
                        report: this.model.reportPristine,
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        user: this.model.user
                    },
                    onHiddenRemove: true                    
                });
                
                $("body").append(this.embedDialog.render().el);
                this.embedDialog.show();

                this.classicUrl.save({ dialog: undefined }, { replaceState: true });
            },
            close: function() {
                window.location = route.reports(this.model.application.get("root"), this.model.application.get("locale"), this.model.application.get("app"));
            }
        });
    }
);