define(
    [
        'backbone',
        'underscore',
        'models/SplunkDBase'
    ],
    function(Backbone, _, SplunkDBaseModel) {
        var RAW_REX = /\$_raw\$/g;
        return SplunkDBaseModel.extend({
            url: "data/ui/workflow-actions",
            initialize: function() {
                SplunkDBaseModel.prototype.initialize.apply(this, arguments);
            },
            /**
             *  $field$ 
             */
            fieldSubstitute: function(text, data) {
                var settings = {
                        interpolate : /\$([\s\S]+?)\$/g 
                    },
                    $matches$ = text.match(settings.interpolate);
                
                text = this.convertRaw(text, data);
                    
                //underscore templates will raise if the value to be replaced
                //is undefined.  Like 5.0, we will strip out $ delimited values
                //if they do not exist in event.  
                if($matches$) { 
                    _($matches$).each(function($string$) {
                        if(!data[$string$.match(/\$([\s\S]+?)\$/)[1]]){
                            text = text.replace($string$, '');
                        }
                    },this);
                }
                return _.template(text, data, settings);
            },
            /**
             * $@sid$ $@offset$, $@namespace$, $@latest_time$, $@field_name$, $@field_value$ 
             */
            systemSubstitute: function(text, sid, offset, namespace, latest_time, fieldName, fieldValue) {
                var settings = {
                        interpolate : /\$@([\s\S]+?)\$/g
                    },
                    data = {
                        sid: sid,
                        offset: offset,
                        namespace: namespace,
                        latest_time: latest_time,
                        field_name: fieldName,
                        field_value: fieldValue
                    };

                return _.template(text, data, settings);
            },
            convertRaw: function(text, data) {
                /*
                 * SPL-77129, when we request results with segmentation the 
                 * _raw field is returned as an object.  Ensure that we 
                 * replace _raw with the _raw string before passing to the template.
                 */
                 if(RAW_REX.test(text) && !_(data._raw).isArray()) {
                    return text.replace(RAW_REX, data._raw.value);
                 }
                
                return text;
            },
            isInFieldMenu: function() {
                return (this.entry.content.get('display_location') != 'event_menu');
            },
            isInEventMenu: function() {
                return (this.entry.content.get('display_location') != 'field_menu');
            },
            isRestrictedByPresenceOfAllFields: function(event) {
                var eventFields   = event.toJSON(),
                    fieldsPresent = 0,
                    fields = _(this.entry.content.get('fields').split(',')).map(function(field) { 
                        return $.trim(field); 
                    },this),
                    fieldsMatching = new Array(fields.length),
                    len = fields.length;

                /*
                 * Iterate through all fields in the event until all specified fields in the workflow
                 * action have been found to be present or not.
                 */ 
                for (var field in eventFields) {
                    
                    //iteration over fields specified in workflow action
                    for(var k=0; k<len; k++) {
                        var rex = this.globber(fields[k]);
                        
                        if(rex) {
                            var value = rex.exec(field);
                            if(!fieldsMatching[k] && value && value.length > 0) {
                                fieldsMatching[k] = true;
                                fieldsPresent++;
                                
                                if(fieldsPresent >= len) {
                                    return false;
                                } 
                            }
                        }
                    }
                }
                return true;
            }, 
            isRestrictedByEventtype: function(event) {
                var eventtypes, eventTypeFromEvent;
                
                if(this.entry.content.has('eventtypes')) {
                    
                    eventtypes = _((this.entry.content.get('eventtypes')).split(',')).map(function(eventtype) {
                        return $.trim(eventtype);
                    },this);
                    
                    if(event.has('eventtype')) {
                        eventTypeFromEvent = event.get('eventtype');
                        
                        var eventtypesLen = eventtypes.length,
                            j = 0;

                        for(j; j<eventtypesLen; j++) {
                            var rex = this.globber(eventtypes[j]);
                            if(rex) {
                                
                                var len = eventTypeFromEvent.length,
                                    i = 0;
                                for(i; i<len; i++) {
                                    var value = rex.exec(eventTypeFromEvent[i]);
                                    if(value && value.length > 0) {
                                        return false;
                                    }
                                }
                            }
                        }
                    } else {
                        if(eventtypes.indexOf('*') > -1) {
                            return false;
                        }
                    }

                } else {
                    return false;
                }

                return true;
            },
            globber: function(str) {
                var rex, 
                    chars = str.split('');


                //build rex for exact match
                if(_.first(chars) !== '*') {
                    chars.unshift('^');
                }

                if(_.last(chars) !== '*') {
                    chars.push('$');
                }
                str = chars.join('').replace(/\*/g, '.*');
                
                try { 
                    rex = new RegExp(str); 
                } catch(e) {}

                return rex;
            }
        });
    }
);
