define(
    [
        'underscore',
        'jquery',
        'backbone',
        'routers/Base',
        'controllers/Report',
        'models/classicurl',
        'models/services/data/ui/Pref',
        'collections/services/data/ui/Times',
        'collections/services/authorization/Roles',
        'collections/services/configs/EventRenderers',
        'collections/services/data/ui/WorkflowActions',
        'util/splunkd_utils'
    ],
    function(
        _,
        $,
        Backbone,
        BaseRouter,
        ReportController,
        classicurl,
        UIPrefsModel,
        TimesCollection,
        RolesCollection,
        EventRenderers,
        WorkflowActionsCollection,
        splunkd_utils
    ) {
        return BaseRouter.extend({
            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.fetchUser = true;
                this.fetchAppLocals = true;

                //controllers
                this.timelineController;

                //models
                this.model = this.model || {};
                this.model.application = this.model.application;
                this.model.uiPrefs = new UIPrefsModel();

                //collections
                this.collection = this.collection || {};
                this.collection.times = new TimesCollection();
                this.collection.roles = new RolesCollection();
                this.collection.eventRenderers = new EventRenderers();
                this.collection.workflowActions = new WorkflowActionsCollection();

                //deferreds
                this.deferreds = this.deferreds || {};
                this.deferreds.roles = this.collection.roles.fetch();
            },
            //Our only Action method
            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);
                
                if (!this.deferreds.uiPrefs) {
                    this.deferreds.uiPrefs = $.Deferred();
                    this.model.uiPrefs.bootstrap(this.deferreds.uiPrefs, this.model.application.get("page"), this.model.application.get("app"), this.model.application.get("owner"));
                }

                if (!this.deferreds.times) {
                    this.deferreds.times = this.collection.times.fetch({
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner")
                        }
                    });
                }

                if (!this.deferreds.eventRenderers) {
                    this.deferreds.eventRenderers = this.collection.eventRenderers.fetch({
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner"),
                            sort_key: 'priority',
                            sort_dir: 'desc',
                            count: -1
                        }
                    });
                }
                
                if (!this.deferreds.workflowActions) {
                    this.deferreds.workflowActions = this.collection.workflowActions.fetch({
                        data: {
                            app: this.model.application.get("app"),
                            owner: this.model.application.get("owner"),
                            count: -1,
                            sort_key: "name"                            
                        }
                    });
                }
                
                this.deferreds.pageViewRendered.done(function(){
                    if (this.shouldRender) {
                        this.pageView.$('.section-padded').remove();//remove once all pages migrated to Page view class correctly
                    }                    
                
                    //this is a longlived singleton object and we have to clear it each time the route is hit
                    classicurl.clear();
    
                    var reportController = new ReportController({
                        model: {
                            application: this.model.application,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            uiPrefs: this.model.uiPrefs
                        },
                        collection: {
                            times: this.collection.times,
                            roles: this.collection.roles,
                            eventRenderers: this.collection.eventRenderers,
                            workflowActions: this.collection.workflowActions
                        },
                        deferreds: {
                            appLocal: this.deferreds.appLocal,
                            user: this.deferreds.user,
                            times: this.deferreds.times,
                            roles: this.deferreds.roles,
                            eventRenderers: this.deferreds.eventRenderers,
                            workflowActions: this.deferreds.workflowActions,
                            uiPrefs: this.deferreds.uiPrefs
                        }
                    });
    
                    reportController.on('ready', function(report) {
                        this.setPageTitle(report.entry.get('name') || _('Report').t());
                        //perform your replace
                        if (this.reportController) {
                            this.reportController.remove();
                        }
                        this.reportController = reportController;
                        $('.preload').replaceWith(this.pageView.el);
                        $('.main-section-body').html(this.reportController.replacementDOM);
                        this.reportController.reflow();
    
                        //replace with application level event view:append
                        $(document).trigger("rendered");
                    }, this);
                }.bind(this));
            }
        });
    }
);
