define(
    [
        'underscore',
        'jquery',
        'backbone',
        'routers/BootstrapSearch',
        'views/search/Master',
        'util/splunkd_utils',
        'splunk.util'
    ],
    function(
        _,
        $,
        Backbone,
        BootstrapSearch,
        SearchView,
        splunkd_utils,
        splunkUtils
    ) {
        return BootstrapSearch.extend({
            initialize: function() {
                BootstrapSearch.prototype.initialize.apply(this, arguments);
                this.setPageTitle(_('Search').t());
                
                //views
                this.searchView = new SearchView({
                    model: {
                        application: this.model.application,
                        appLocal: this.model.appLocal,
                        chartWorking: this.model.chartWorking,
                        classicUrl: this.model.classicUrl,
                        metaDataJob: this.model.metaDataJob,
                        metaDataResult: this.model.metaDataResult,
                        report: this.model.report,
                        reportPristine: this.model.reportPristine,
                        result: this.model.result,
                        resultJsonCols: this.model.resultJsonCols,
                        resultJsonRows: this.model.resultJsonRows,
                        searchJob: this.model.searchJob,
                        summary: this.model.summary,
                        timeRange: this.model.timeRange,
                        timeline: this.model.timeline,
                        user: this.model.user,
                        uiPrefs: this.model.uiPrefs
                    },
                    collection: {
                        times: this.collection.times,
                        eventRenderers: this.collection.eventRenderers,
                        workflowActions: this.collection.workflowActions,
                        selectedFields: this.collection.selectedFields
                    },
                    deferreds: {
                        appLocal: this.deferreds.appLocal,
                        user: this.deferreds.user,
                        times: this.deferreds.times,
                        eventRenderers: this.deferreds.eventRenderers,
                        workflowActions: this.deferreds.workflowActions,
                        uiPrefs: this.deferreds.uiPrefs
                    }
                });
                
                $(window).resize(_.debounce(function(){
                    this.setBodyMinHeight();
                }.bind(this), 0));
            },
            //Our only Action method
            page: function(locale, app, page) {
                BootstrapSearch.prototype.page.apply(this, arguments);
        
                $.when(this.baseDeactivateDeferred, this.deferreds.times, this.deferreds.pageViewRendered).then(function(){
                    if (this.shouldRender) {
                        //insert the top bars
                        //this.pageView.$('.section-padded').remove();//remove once all pages migrated to Page view class correctly
                        $('.preload').replaceWith(this.pageView.el);
                        this.deferreds.preloadReplaced.resolve();
                    }
                }.bind(this));
                
                $.when(this.baseActivateDeferred).then(function() {
                    this.searchView.activate({showAutoPause: this.showAutoPause});
                    this.activate();
                    
                    if (this.shouldRender) {
                        this.searchView.render().replaceContentsOf($('.main-section-body'));
                        //this.searchView.reflow();
                        //replace with application level event view:append
                        $(document).trigger("rendered");
                    }
                }.bind(this));
            },
            activate: function() {
                this.model.report.entry.content.set({
                    'disable_timerange_trigger': true
                });
                
                //Events for the report model
                this.model.report.entry.content.on('change:display.prefs.events.offset change:display.events.fields change:display.events.maxLines change:display.events.raw.drilldown change:display.events.list.drilldown change:display.prefs.events.count change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.general.type change:display.events.table.sortColumn change:display.events.table.sortDirection', _.debounce(function() {
                    this.fetchResult();
                }, 0), this);
                
                this.model.result.on('forcefetch', function() {
                    this.fetchResult();
                },this);
                
                this.model.report.entry.content.on('change:display.events.type', function(m, value, options) {
                    if(m.previousAttributes()['display.events.type'] == 'table' || m.get('display.events.type') === 'table') {
                        this.fetchResult();
                    }
                }, this);
                
                this.model.summary.on('change:fieldPickerOpen', this.fetchSummary, this);
                
                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime', function() {
                    this.model.report.entry.content.set('display.prefs.events.offset', '0');
                }, this);
                
                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.general.type', _.debounce(function() {
                    this.fetchSummary();
                }, 0), this);
                
                this.model.report.entry.content.on('change:display.events.timelineEarliestTime change:display.events.timelineLatestTime change:display.general.type', _.debounce(function() {
                    this.fetchTimeline();
                }, 0), this);
                
                this.model.report.entry.content.on('change:display.general.type', this.fetchResultJSONColumn, this);
                
                this.model.report.entry.content.on('change:display.prefs.events.count', function(){
                    if (this.model.report.entry.content.get('display.prefs.events.count')){
                        this.model.report.entry.content.set({'display.prefs.events.offset': '0'});
                    }
                }, this);

                this.model.report.entry.content.on('change:display.prefs.statistics.count', function(){
                    if (this.model.report.entry.content.get('display.prefs.statistics.count')){
                        this.model.report.entry.content.set({'display.prefs.statistics.offset': '0'});
                    }
                }, this);
                
                this.model.report.entry.content.on('change:display.general.enablePreview', function(){
                    if (splunkUtils.normalizeBoolean(this.model.report.entry.content.get('display.general.enablePreview'))) {
                        this.model.searchJob.enablePreview();
                    } else {
                        this.model.searchJob.disablePreview();
                    }
                }, this);

                this.model.report.entry.content.on('change:display.statistics.sortColumn change:display.statistics.sortDirection change:display.prefs.statistics.count change:display.prefs.statistics.offset change:display.general.type', _.debounce(function() {
                    this.fetchResultJSONRows();
                }, 0), this);
                
                //arbitrators to determine when to push on the history stack
                this.model.report.entry.content.on('change', function(){
                    var changed = $.extend(true, {}, this.model.report.entry.content.changedAttributes()),
                        jobPopulationDeferred = $.Deferred(),
                        uiPrefsPopulationDeferred = $.Deferred(),
                        options = {};
                    
                    this.populateJob(jobPopulationDeferred, changed);
                    this.populateUIPrefs(uiPrefsPopulationDeferred, changed);
                    
                    $.when(jobPopulationDeferred, uiPrefsPopulationDeferred).always(function() {
                        if (this.model.report.id && (this.model.report.id !== this.model.reportPristine.id)) {
                            this.model.searchJob.set("cannotPauseOnRemove", true);
                            options.forceAttrs = {s: this.model.report.id};
                            options.forceTrigger = true;
                        }
                        this.populateClassicUrl(changed, options);
                    }.bind(this));
                }, this);
                
                this.model.report.on('change:id', function(){
                    this.model.searchJob.set("cannotPauseOnRemove", true);
                    this.model.classicUrl.save(
                        {
                            's': this.model.report.id
                         },
                         {
                             trigger: true
                         }
                    );
                }, this);
                
                this.model.report.on("close", this.close, this);
                
                //Events on the searchJob model
                this.model.searchJob.on('destroy', function() {
                    this.model.classicUrl.save(
                        {
                            "display.prefs.events.offset": undefined,
                            "display.prefs.statistics.offset": undefined,
                            "display.events.timelineEarliestTime": undefined,
                            "display.events.timelineLatestTime": undefined,
                            "display.statistics.sortColumn": undefined,
                            "display.statistics.sortDirection": undefined,
                            "display.events.table.sortColumn": undefined,
                            "display.events.table.sortDirection": undefined,
                            auto_pause: undefined
                        },
                        {
                            replaceState: true
                        }
                    );
                }, this);

                // If the job is prepared, it should have enough information to apply any search-string-based defaults.
                // If not, these updates will be applied when the "prepared" event is fired.
                if(!this.model.searchJob.isNew() && !this.model.searchJob.isPreparing()) {
                    this.updateReportBasedOnSearchString();
                }
                this.model.searchJob.on("prepared", function(){
                    this.updateReportBasedOnSearchString();
                    this.registerSearchJobFriends();
                }, this);
                
                this.model.searchJob.on("close", this.close, this);
            },
            deactivate: function() {
                if (!this.shouldRender) {
                    this.model.report.off(null, null, this);
                    this.model.report.entry.content.off(null, null, this);
                    this.model.result.off(null, null, this);
                    this.model.summary.off(null, null, this);
                    this.model.searchJob.off(null, null, this);
                    this.collection.selectedFields.off(null, null, this);
                }
                this.searchView.deactivate({deep: true});
                BootstrapSearch.prototype.deactivate.apply(this, arguments);
            },
            fetchResult: function() {
                if (this.model.result.id && this.model.report.entry.content.get('display.general.type')==='events') {
                    BootstrapSearch.prototype.fetchResult.apply(this);
                }
            },
            fetchResultJSONColumn: function() {
                if (this.model.resultJsonCols.id && this.model.report.entry.content.get('display.general.type')==='visualizations') {
                    BootstrapSearch.prototype.fetchResultJSONColumn.apply(this);
                }
            },
            fetchResultJSONRows: function() {
                var type = this.model.report.entry.content.get('display.general.type');
                if (this.model.resultJsonRows.id && (type === 'visualizations' || type === 'statistics')) {
                    BootstrapSearch.prototype.fetchResultJSONRows.apply(this);
                }
            },
            fetchSummary: function() {
                if (this.model.summary.id && this.model.report.entry.content.get('display.general.type')==='events') { 
                    if (!this.model.summary.get('fieldPickerOpen')) {
                        BootstrapSearch.prototype.fetchSummary.apply(this);
                    }
                }
            },
            fetchTimeline: function() {
                if (this.model.timeline.id && this.model.report.entry.content.get('display.general.type')==='events') {
                    BootstrapSearch.prototype.fetchTimeline.apply(this);
                }
            },
            setBodyMinHeight: function() {
                this.minHeight = this.pageView.$('header').height() + $(window).height();
                $('body').css('min-height', this.minHeight);
            },
            updateReportBasedOnSearchString: function() {
                var reportSearch = this.model.searchJob.entry.content.get('reportSearch'),
                    searchSpecificDefaults = this.model.report.getSearchSpecificDefaults(reportSearch);

                // If there is no explicit general type in the permalink, and no custom general type defined in the job,
                // it is safe to apply the search-specific defaults.
                if(!this.model.classicUrl.has('display.general.type') &&
                        !this.model.searchJob.entry.content.custom.get('display.general.type') &&
                        !_(searchSpecificDefaults).isEmpty()) {
                    this.model.report.entry.content.set(searchSpecificDefaults);
                }
                // Call setDisplayType on the report to make sure the transforming/non-transforming experience is
                // handled correctly.
                this.model.report.setDisplayType(this.model.searchJob.isReportSearch(), reportSearch);
            }
        });
    }
);
