define(
    [
        'jquery',
        'underscore',
        'models/classicurl',
        'models/services/data/vix/IndexActions',
        'collections/services/data/vix/Providers',
        'collections/services/data/vix/Indexes',
        'collections/services/configs/Indexes',
        'collections/services/admin/conf-impersonations',
        'collections/services/authentication/Users',
        'views/virtual_indexes/Master',
        'views/virtual_indexes/NoLicense',
        'routers/Base'
    ],
    function(
        $,
        _,
        classicurlModel,
        IndexActions,
        ProviderCollection,
        IndexCollection,
        IndexesConfCollection,
        UserImpersonationCollection,
        SplunkUsers,
        VixMasterView,
        NoLicenseView,
        BaseRouter
        ) {
        return BaseRouter.extend({

            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.setPageTitle(_('Virtual indexes').t());
                this.enableAppBar = false;
                var that = this;
                this.pageReadydfd = $.Deferred();
                this.indexesConfCollection = new IndexesConfCollection();
                this.indexesConfCollection.fetch({
                    data: {
                        search: 'name=provider-family*',
                        f: 'name'
                    }
                }).done(function() {
                    if (that.indexesConfCollection.length > 0) {
                        that.loadMainPage();
                    } else {
                        that.loadNoLicense();
                    }
                    that.pageReadydfd.resolve();
                });
            },

            loadMainPage: function() {
                this.providers = new ProviderCollection();
                this.indexes = new IndexCollection();
                this.providersAll = new ProviderCollection();
                this.indexesAll = new IndexCollection();
                this.indexActions = new IndexActions();
                this.userImpersonationCollection = new UserImpersonationCollection();
                this.splunkUsers = new SplunkUsers();

                this.providersDfd = $.Deferred();
                this.userImpersonationDfd = $.Deferred();
                this.splunkUsersDfd = $.Deferred();

                this.userImpersonationCollection.fetch().done(function() {
                    this.userImpersonationDfd.resolve();
                }.bind(this));
                this.splunkUsers.fetch().done(function() {
                    this.splunkUsersDfd.resolve();
                }.bind(this));

                this.providers.fetchData.set('count', 10);
                this.indexes.fetchData.set('count', 10);
                this.indexes.fetch();
                this.providers.fetch().done(function() {
                    this.providersDfd.resolve();
                }.bind(this));

                this.indexesAll.fetchData.set('count', -1);
                this.indexesAll.fetchData.set('f', 'name');
                this.providersAll.fetchData.set('count', -1);
                this.providersAll.fetchData.set('f', 'name');
                this.indexesAll.fetch();
                this.providersAll.fetch();

                this.providers.on('deleteApproved', function(victimModel) {
                    var that = this;
                    victimModel.destroy().done(function() {
                        that.providers.fetch();
                        that.providersAll.fetch();
                    });
                }, this);
                this.indexes.on('deleteApproved', function(victimModel) {
                    var that = this;
                    victimModel.destroy().done(function() {
                        that.providers.fetch();
                        that.indexes.fetch();
                        that.indexesAll.fetch();
                    });
                }, this);
                this.indexes.on('disableRequest', function(victimModel) {
                    var that = this,
                        links = victimModel.entry.links;
                    this.indexActions.set('id', links.get('disable'));
                    this.indexActions.save().done(function() {
                        that.providers.fetch();
                        that.indexes.fetch();
                        that.indexesAll.fetch();
                    });
                }, this);
                this.indexes.on('enableRequest', function(victimModel) {
                    var that = this,
                        links = victimModel.entry.links;
                    this.indexActions.set('id', links.get('enable'));
                    this.indexActions.save().done(function() {
                        that.providers.fetch();
                        that.indexes.fetch();
                        that.indexesAll.fetch();
                    });
                }, this);

                this.masterView = new VixMasterView({
                    model: {
                        application: this.model.application
                    },
                    collection: {
                        providers: this.providers,
                        indexes: this.indexes,
                        providersAll: this.providersAll,
                        indexesAll: this.indexesAll,
                        userImpersonation: this.userImpersonationCollection,
                        splunkUsers: this.splunkUsers
                    },
                    providersDfd: this.providersDfd,
                    userImpersonationDfd: this.userImpersonationDfd,
                    splunkUsersDfd: this.splunkUsersDfd
                });
            },

            loadNoLicense: function() {
                this.masterView = new NoLicenseView({
                    model: {
                        application: this.model.application
                    }
                });
            },


            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);

                var that = this;
                this.deferreds.pageViewRendered.done(function() {
                    $('.preload').replaceWith(that.pageView.el);
                });

                $.when(that.pageReadydfd, that.deferreds.pageViewRendered).done(function() {
                    that.pageView.$('.main-section-body').append(that.masterView.render().el);
                });


            }
        });
    }
);
