define(
    [
        'jquery',
        'underscore',
        'routers/Base',
        'models/services/data/vix/Provider',
        'models/classicurl',
        'collections/services/configs/Indexes',
        'views/virtual_indexes/config/ProviderSetup'
    ],
    function(
        $,
        _,
        BaseRouter,
        ProviderModel,
        ClassicurlModel,
        IndexesConfCollection,
        ProviderSetupView
        ) {
        return BaseRouter.extend({
            initialize: function() {
                BaseRouter.prototype.initialize.apply(this, arguments);
                this.enableAppBar = false;
                this.providerModel = new ProviderModel();
                this.indexesConfCollection = new IndexesConfCollection();
                this.indexesConfCollection.fetch({
                    data: {
                        search: 'name=provider-family*',
                        f: 'name'
                    }
                });

                this.providerSetup = new ProviderSetupView({
                    model: {
                        provider: this.providerModel,
                        application: this.model.application
                    },
                    collection: {
                        indexesConf: this.indexesConfCollection
                    }
                });

            },
            page: function(locale, app, page) {
                BaseRouter.prototype.page.apply(this, arguments);
                this.deferreds.pageViewRendered.done(function() {
                    $('.preload').replaceWith(this.pageView.el);
                }.bind(this));

                ClassicurlModel.fetch({
                    success: function(model, response) {
                        var id = ClassicurlModel.get('id');
                        if (id) {
                            this.providerModel.id = id;
                            this.providerModel.fetch();
                            this.providerModel.set({'mode':'edit'}, {silent: true});
                            this.setPageTitle(_('Edit provider').t());
                        } else {
                            // fetch _new with vix.family set to the first available family name to prepopulate the form
                            this.providerModel.set({'mode':'new'}, {silent: true});
                            this.indexesConfCollection.on('change reset', function() {
                                var fetchOptions = {};
                                if (this.indexesConfCollection.length && this.indexesConfCollection.at(0).entry.get('name')) {
                                    var familyName = this.indexesConfCollection.at(0).entry.get('name').match(/provider-family:(.*)/)[1];
                                    fetchOptions = {data:{'vix.family':familyName}};
                                }
                                this.providerModel.fetch(fetchOptions);
                                this.setPageTitle(_('Add new provider').t());
                            }, this);
                        }
                        this.deferreds.pageViewRendered.done(function() {
                            this.pageView.$('.main-section-body').append(this.providerSetup.render().el);
                        }.bind(this));
                    }.bind(this)
                });
            }
        });
    }
);