define(function(require, exports, module) {
    var _ = require('underscore');
    var $ = require('jquery');
    var Select2 = require("select2/select2");

    require("css!contrib/select2-3.3.1/select2.css");
    require("css!../css/select2.css");

    var BaseDropdownViewMixin = {
        options: {
            managerid: null,
            valueField: "",
            labelField: "",
            choices: [],
            selectFirstChoice: false,
            "default": undefined,
            width: 200,
            minimumResultsForSearch: 8,
            showClearButton: true, // allow a selection to be cleared from the dropdown itself
            disabled: false,
            value: undefined
        },

        events: {
            "change select": "onDomChange",
            "selected select": "onDomChange"
        },
        
        _setSelect2Value: function(value) {                
            // We need to update the select2 control, but to do that 
            // properly and handle the cases of value="", we need to compute
            // {id: value, text: label} objects.
            // First, we create these objects for any selected value
            var valueAsArray = this.valueIsList ? value : [value];
            var selectedObject = {};
            _.each(this._vizData, function(row) {
                if (_.contains(valueAsArray, row.value)) {
                    selectedObject[row.value] = $.trim(row.label) || row.value;
                }
            });
            var selectedValues = _(valueAsArray).map(function(val) {
                return {
                    id: val,
                    text: selectedObject[val]
                };
            });

            // Now, we set the value appropriately.
            if (this.valueIsList) {
                this._viz.select2('data', selectedValues);
            }
            else if (!this.valueIsList && selectedValues.length > 0) {
                this._viz.select2('data', selectedValues[0]);
            }
            else {
                this._viz.select2('val', undefined);
            }
        },

        updateDomVal: function() {
            if (this.$('.select2-container.placeholder').length == 0) {
                var value = this.val();

                // Update the raw DOM
                this.$('option:selected').prop('selected', false);
                this._viz.data(value);

                // Update select2
                this._setSelect2Value(value);
            }
        },

        _domVal: function() {
            var value = undefined;
            var valueObject = this._viz.select2('data');
            if (this.valueIsList) {
                value = _.pluck(valueObject, "id");
            }
            else {
                value = valueObject === null ? undefined : valueObject.id;
            }
            return value;
        },

        onDomChange: function(e) {
            // This function can get called when two separate events are triggered:
            // 1. The 'selected' event, which is triggered whenever select2 goes from
            // one value to another, but not to the undefined state.
            // 2. The 'change' event, which is triggered whenever select2 goes from
            // any value to another value, except the from the undefined to a value
            // that is "".
            //
            // The above is due to a shortcoming in Select2, where an option with
            // value="" is seen the same as the "undefined" (placeholder) state.
            // Given this, we register for both values, and use the 'selected'
            // event, except to deduce the value -> undefined transition.
            var value = this._domVal();

            // Unfortunately, the above rules don't quite apply for the case
            // of multiselect, so for that case, we use always use the 'change'
            // event.
            if (this.valueIsList && e.type === "selected") {
                return;
            }

            // If the type is change and the value isn't equal to undefined,
            // then we can stop, because we already handled this in the 'selected'
            // event.
            if (!this.valueIsList && e.type === "change" && value !== undefined) {
                return;
            }

            this.val(value);
        },

        _onDisable: function(e) {
            if (!this._viz) {
                return;
            }

            this._disable(this.settings.get("disabled"));
        },

        _disable: function(state) {
            $(this._viz).select2(state ? "disable" : "enable", true);
        },

        _select2: function(item, options) {
            var selectOptions = _.extend({
                minimumResultsForSearch: parseInt(this.settings.get('minimumResultsForSearch'), 10),
                allowClear: this.settings.get('showClearButton'),
                placeholder: '',
                formatResult: function(item, el) {
                    if (item) {
                        // Update each row in the open dropdown to have a tooltip
                        el.attr('title', item.text);
                        return _.escape(item.text);
                    } else {
                        return item;
                    }
                },
                formatSelection: function(item, el) {
                    if (item) {
                        // Update the select box itself to have a tooltip
                        el.attr('title', item.text);
                        return item.text;
                    } else {
                        return item;
                    }
                }
            }, options || {});
            item.select2("close");
            item.select2(selectOptions);
        },

        createView: function() {            
            var select = $(this.selectRoot);
            this.$el.html(select);
            select.width(this.settings.get('width'));
            this._select2(select);
            return select;
        },

        updateView: function(viz, data) {
            viz.empty();
            this._vizData = data;

            if (data.length === 0) {
                this._select2(viz, {placeholder: ''});
                return this;
            }
            
            var hasEmptyValueOption = _.any(data, function(d) { return (d.value === ""); });

            // Select2 requires an empty <option/> element to be present
            // if the 'placeholder' and 'allowClear' options are set. We don't
            // want it to clash with any user-specified options, so we give it
            // a particular value (but no label, so it is still empty from 
            // Select2's point of view) if we already have an option with an
            // empty value.
            viz.append($("<option/>").attr("value", hasEmptyValueOption ? "__placeholder" : undefined));

            // Get the actual value of the control
            var controlValue = this.valueIsList ? this.settings.get("value") : [this.settings.get("value")];

            _.each(data, function(row) {
                // Create the <option> tag for this entry and ensure we sync
                // up the display value if it is the currently selected one.
                var option = $("<option/>").text($.trim(row.label) || row.value).attr('value', row.value);
                if (_.contains(controlValue, row.value)) {
                    option.attr('selected', 'selected');
                }
                viz.append(option);
            });
            
            this.updateDomVal();
            
            return this;
        }
    };
    return BaseDropdownViewMixin;
});
