define(function (require, exports, module) {
    var mvc = require('./mvc'),
        BaseSplunkView = require("./basesplunkview"),
        GlobalNav = require('views/shared/splunkbar/Master'),
        AppNav = require('views/shared/appbar/Master'),
        sharedModels = require('./sharedmodels'),
        userAgent = require('helpers/user_agent'),
        config = require('models/config');


    var HeaderView = BaseSplunkView.extend({
        moduleId: module.id,
        
        className: 'splunk-header',

        options: {
            appbar: true,
            acceleratedAppNav: false,
            useSessionStorageCache: false
        },

        initialize: function() {
            this.configure();
            this.model = this.model || {};
            this.model.application= sharedModels.get("app");
            this.model.user= sharedModels.get("user");
            this.model.appLocal= sharedModels.get("appLocal");
            this.model.serverInfo= sharedModels.get("serverInfo");

            var acceleratedAppNav = this.settings.get('acceleratedAppNav') && userAgent.isIE7();
            this.collection = this.collection || {};
            this.collection.appLocals = sharedModels.get("appLocals");

            this.globalNav = GlobalNav.create({
                model: {
                    application: this.model.application,
                    appLocal: this.model.appLocal,
                    user: this.model.user,
                    serverInfo: this.model.serverInfo,
                    config: config
                },
                collection: {
                    apps: this.collection.appLocals
                }
            });

            if(this.settings.get('appbar')){
                this.appNav = AppNav.create({
                    model: {
                        application: this.model.application,
                        app: this.model.appLocal
                    },
                    appServerUrl: acceleratedAppNav,
                    useSessionStorageCache: this.settings.get('useSessionStorageCache'),
                    autoRender: false
                });
            }
        },
        render: function() {
            this.$el.empty().append(this.globalNav.render().el);
            if(this.settings.get('appbar')){
                this.$el.append(this.appNav.render().el);
            }
            return this;
        }
    });

    return HeaderView;
});
