define(function(require) {
    var _ = require('underscore'),
            $ = require('jquery'),
            Backbone = require('backbone');

    /**
     * The dashboard row view is a delegate view which handles the layout of dashboard panels within a row. Actions are
     * executed when panels are shown, hidden or removed.
     */
    var DashboardRowView = Backbone.View.extend({
        initialize: function() {
            this.listenTo(this.model, 'change:edit', this.onEditStateChange, this);
        },
        onEditStateChange: function() {
            this.$el.off('DOMSubtreeModified');
            if((!this.model.get('edit')) && this.$('.dashboard-panel').length > 1) {
                var fn = _.debounce(this.alignItemHeights.bind(this), 1000);
                this.$el.bind('DOMSubtreeModified', fn);
                fn();
            }
        },
        onContentChange: function() {
            this.$el.off('DOMSubtreeModified');
            var cells = this.$('.dashboard-cell');
            if(cells.length === 0) {
                return this.remove();
            }
            cells.removeClass('last-visible');
            if (!this.model.get('edit')) {
                cells = _(cells).filter(function(el) {
                    return $(el).find('.dashboard-element:not(.hidden)').length > 0 || !!$.trim($(el).find('.fieldset').html());
                });
                if (cells.length > 0) {
                    $(cells).last().addClass('last-visible');
                }
            }
            $(cells).css({ width: String(100 / cells.length) + '%' }).find('.panel-element-row').each(function() {
                var elements = $(this).find('.dashboard-element');
                elements.css({ width: String(100 / elements.length) + '%' });
            });
            if(cells.length > 1) {
                this.alignItemHeights();
            }
            this.onEditStateChange();
        },
        onPanelVisibilityChange: function(){
            this.onContentChange();
            // Force charts to re-render since widths of the panels have probably changed
            $(window).trigger('resize');
        },
        alignItemHeights: function() {
            var row = this.$el, items = row.find('.dashboard-panel');
            items.css({ 'min-height': 0 }).css({
                'min-height': _.max(_.map(items, function(i) { return $(i).height(); }))
            });
        },
        render: function() {
            this.onContentChange();
            this.$el.off('cellRemoved');
            this.$el.off('panelVisibilityChanged');
            this.$el.on('cellRemoved', _.bind(this.onContentChange, this));
            this.$el.on('panelVisibilityChanged', _.bind(this.onPanelVisibilityChange, this));
            return this;
        }
    });

    return DashboardRowView;
});
