define(
    [
        'module',
        'underscore',
        'jquery',
        'backbone',
        'views/Base',
        'views/shared/tablecaption/Master',
        'views/shared/TableHead',
        'views/shared/delegates/TableRowToggle',
        'views/alerts/table/TableRow',
        'views/alerts/table/MoreInfo',
        'views/shared/delegates/TableDock'
    ],
    function(
        module,
        _,
        $,
        Backbone,
        BaseView,
        CaptionView,
        TableHeadView,
        TableRowToggleView,
        TableRowView,
        MoreInfoView,
        TableDock
    )
    {
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *          state: <models.Base>,
             *          application: <models.Application>,
             *          appLocal: <models.services.AppLocal>>,
             *          uiPrefs: <models.services.admin.UIPrefs>
             *          user: <models.services.admin.User>
             *     },
             *     collection: {
             *         savedAlerts: <collections.services.SavedSearches>,
             *         roles: <collections.services.authorization.Roles>
             *     }
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.children.tableRowToggle = new TableRowToggleView({el: this.el, collapseOthers: true });

                this.children.caption = new CaptionView({
                    countLabel: _('Alerts').t(),
                    model: {
                        state: this.model.state,
                        application: this.model.application,
                        uiPrefs: this.model.uiPrefs
                    },
                    collection: this.collection.savedAlerts,
                    noFilter: false
                });

                this.children.head = new TableHeadView({
                    model: this.model.state,
                    columns: [
                        {label: 'i', className: 'col-info', html: '<i class="icon-info"></i>'},
                        {label: _('Title').t(), sortKey: 'name'},
                        {label: _('Actions').t(), className: 'col-actions'},
                        {label: _('Owner').t(), sortKey: 'eai:acl.owner,name', className: 'col-owner'},
                        {label: _('App').t(), sortKey: 'eai:acl.app,name', className: 'col-app'},
                        {label: _('Sharing').t(), sortKey: 'eai:acl.sharing,name', className: 'col-sharing'}
                    ]
                });

                this.children.rows = this.rowsFromCollection();

                this.children.tableDock = new TableDock({ el: this.el, offset: 36, dockScrollBar: false, defaultLayout: 'fixed', flexWidthColumn: 1  });
                this.activate();
            },
            startListening: function () {
                this.listenTo(this.collection.savedAlerts, 'reset', this.renderRows);
            },
            rowsFromCollection: function() {
                return _.flatten(
                    this.collection.savedAlerts.map(function(model, i) {
                        return [
                            new TableRowView({
                                model: {
                                    savedAlert: model,
                                    application: this.model.application,
                                    state: this.model.state,
                                    appLocal: this.model.appLocal,
                                    user: this.model.user
                                },
                                collection: this.collection.roles,
                                index: i
                            }),
                            new MoreInfoView({
                                model: {
                                    savedAlert: model,
                                    application: this.model.application,
                                    appLocal: this.model.appLocal,
                                    user: this.model.user
                                },
                                collection: {
                                    roles: this.collection.roles
                                },
                                index: i
                            })
                        ];
                    }, this)
                );
            },
            _render: function() {
                _(this.children.rows).each(function(row){
                    row.render().appendTo(this.$('.alerts-listings'));
                }, this);
                this.children.tableDock.update();
            },
            renderRows: function() {
                 _(this.children.rows).each(function(row){ row.remove(); }, this);
                 this.children.rows = this.rowsFromCollection();
                 this._render();
            },
            render: function() {
                if (!this.el.innerHTML) {
                    this.children.caption.render().appendTo(this.$el);
                    this.$el.append(this.compiledTemplate({}));
                    this.children.head.render().prependTo(this.$('> .table-chrome'));
                }
                this._render();
                return this;
            },
            template: '\
                <table class="table table-chrome table-striped table-row-expanding">\
                <tbody class="alerts-listings"></tbody>\
                </table>\
            '
        });
    }
);
