define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/dashboards/Title',
        'views/dashboards/table/Master'
    ],
    function(
        _,
        module,
        BaseView,
        TitleView,
        TableView
    ){
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         state: <models.Base>,
             *         application: <models.Application>,
             *         appLocal: <models.services.AppLocal>,
             *         uiPrefs: <models.services.admin.UIPrefs>
             *     },
             *     collection: {
             *         dashboards: <collections.Dashboards>,
             *         roles: <collections.services.authorization.Roles>
             *     }
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.children.title = new TitleView({
                    model: {
                        application: this.model.application
                    },
                    collection: this.collection.dashboards
                });
                this.children.table = new TableView({
                    model: {
                        state: this.model.state,
                        application: this.model.application,
                        uiPrefs: this.model.uiPrefs,
                        user: this.model.user, 
                        appLocal: this.model.appLocal 
                    },
                    collection: {
                        dashboards: this.collection.dashboards,
                        roles: this.collection.roles
                    }
                });
            },
            render: function() {
                this.$el.html(this.template);
                var $inner = this.$('.section-header');
                $inner.append(this.children.title.render().el);
                this.$el.append(this.children.table.render().el);
                return this;
            },
            template: '<div class="section-padded section-header"></div><div class="divider"></div>'
        });
    }
);
