define(
    [
        'module',
        'underscore',
        'jquery',
        'backbone',
        'views/Base',
        'views/shared/tablecaption/Master',
        'views/shared/TableHead',
        'views/shared/delegates/TableRowToggle',
        'views/dashboards/table/TableRow',
        'views/dashboards/table/MoreInfo',
        'collections/services/authorization/Roles',
        'views/shared/delegates/TableDock',
        'models/services/ScheduledView'
    ],
    function(
        module,
        _,
        $,
        Backbone,
        BaseView,
        CaptionView,
        TableHeadView,
        TableRowToggleView,
        TableRow,
        MoreInfo,
        RolesCollection,
        TableDock,
        ScheduledView
    )
    {
        return BaseView.extend({
            moduleId: module.id,
            /**
             * @param {Object} options {
             *     model: {
             *         state: <models.State>,
             *         application: <models.Application>,
             *         uiPrefs: <models.services.admin.UIPrefs>,
             *         user: <models.service.admin.user>
             *     }, 
             *     collection: {
             *         dashboards: <collections.Dashboards>,
             *         roles: <collections.services.authorization.Roles>
             *     }
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.children.tableRowToggle = new TableRowToggleView({el: this.el, collapseOthers: true });

                this.children.caption = new CaptionView({
                    countLabel: _("Dashboards").t(),
                    model: {
                        state: this.model.state,
                        application: this.model.application,
                        uiPrefs: this.model.uiPrefs
                    },
                    collection: this.collection.dashboards,
                    noFilter: false,
                    filterKey: ['label','eai:data']
                });

                this.children.head = new TableHeadView({
                    model: this.model.state,
                    columns: [
                        { label: 'i', className: 'col-info', html: '<i class="icon-info"></i>'},
                        { label: _("Title").t(), sortKey: 'label'},
                        { label: _("Actions").t(), className: 'col-actions'},
                        { label: _("Owner").t(), sortKey: 'eai:acl.owner,label', className: 'col-owner'},
                        { label: _("App").t(), sortKey: 'eai:acl.app,label', className: 'col-app'},
                        { label: _("Sharing").t(), sortKey: 'eai:acl.sharing,label', className: 'col-sharing'}
                    ]
                });
                this.children.rows = this.rowsFromCollection();
                this.children.tableDock = new TableDock({
                    el: this.el,
                    offset: 36,
                    dockScrollBar: false,
                    defaultLayout: 'fixed',
                    flexWidthColumn: 1
                });
                this.collection.dashboards.on('reset remove', _.debounce(this.renderRows), this);
            },
            rowsFromCollection: function() {
                return _.flatten(
                    this.collection.dashboards.map(function(model, i) {
                        var scheduledView = new ScheduledView();
                        return [
                            new TableRow({
                                model: {
                                    state: this.model.state,
                                    dashboard: model,
                                    application: this.model.application,
                                    scheduledView: scheduledView,
                                    user: this.model.user,
                                    appLocal: this.model.appLocal 
                                },
                                collection: this.collection,
                                index: i
                            }),
                            new MoreInfo({
                                model: {
                                    state: this.model.state,
                                    dashboard: model,
                                    application: this.model.application,
                                    scheduledView: scheduledView, 
                                    user: this.model.user,
                                    appLocal: this.model.appLocal
                                },
                                collection: this.collection.roles,
                                index: i
                            })
                        ];
                    }, this)
                );
            },
            _render: function() {
                _(this.children.rows).each(function(row){
                    this.$('.dashboards-listings').append(row.render().el);
                }, this);
                this.children.tableDock.update();
            },
            renderRows: function() {
                _(this.children.rows).invoke('remove');
                this.children.rows = this.rowsFromCollection();
                this._render();
            },
            render: function() {
                if (!this.el.innerHTML) {
                    this.$el.append(this.children.caption.render().el);
                    this.$el.append(this.compiledTemplate({}));
                    this.$('> .table-chrome').prepend(this.children.head.render().el);
                }
                this._render();
                return this;
            },
            template: '\
                <table class="table table-chrome table-striped table-row-expanding">\
                <tbody class="dashboards-listings"></tbody>\
                </table>\
            '
        });
    }
);
