define(['underscore', 'module', 'views/Base', 'splunk.i18n'], function(_, module, Base, i18n) {
     return Base.extend({
        tagName: 'li',
        moduleId: module.id,
        initialize: function(){
            Base.prototype.initialize.apply(this, arguments);
        },
        startListening: function() {
            this.listenTo(this.model.searchJob.entry.content, 'change:eventCount change:dispatchState', this.debouncedRender);
            this.listenTo(this.model.timeline.buckets, 'reset', this.render);
            this.listenTo(this.model.report.entry.content, 'change:display.general.type', this.toggleActive);
        },
        activate: function(options) {
            if (this.active) {
                return Base.prototype.activate.apply(this, arguments);
            }
            this.render();
            return Base.prototype.activate.apply(this, arguments);
        },
        events: {
            'click a': function(e){
                this.model.report.entry.content.set('display.general.type', $(e.currentTarget).attr('data-type'));
                e.preventDefault();
            }
        },
        toggleActive: function() {
            (this.model.report.entry.content.get('display.general.type') == 'events') ?
                this.$el.addClass('active') :
                this.$el.removeClass('active');
        },
        render: function() {
            var length = this.model.searchJob.entry.content.get("eventCount") || 0,
                timelineEarliestTime = this.model.report.entry.content.get('display.events.timelineEarliestTime'),
                timelineLatestTime = this.model.report.entry.content.get('display.events.timelineLatestTime');
            
            if (this.model.searchJob.entry.content.get('statusBuckets')>0 && timelineEarliestTime && timelineLatestTime) {
                length = this.model.timeline.totalCount(timelineEarliestTime, timelineLatestTime);
            }
           
            this.$el.html(this.compiledTemplate({
                _: _,
                length: length,
                searchJob: this.model.searchJob,
                i18n: i18n
            }));
            this.toggleActive();
            return this;
        },
        template:'\
            <a href="#" data-type="events">\
                <% if (searchJob.isNew() || searchJob.entry.content.get("statusBuckets") === 0) { %>\
                    <%- _("Events").t() %>\
                <% } else { %>\
                    <%- _("Events").t() %> (<%= i18n.format_decimal(length) %>)\
                <% } %>\
            </a>\
        '
    });
});
