define(['underscore', 'module', 'views/Base', 'splunk.i18n'], function(_, module, Base, i18n) {
    return Base.extend({
        tagName: 'li',
        moduleId: module.id,
        initialize: function(){
            Base.prototype.initialize.apply(this, arguments);
        },
        startListening: function() {
            this.listenTo(this.model.searchJob.entry.content, 'change:resultPreviewCount change:resultCount change:dispatchState', this.debouncedRender);
            this.listenTo(this.model.report.entry.content, 'change:display.general.type', this.toggleActive);
        },
        activate: function(options) {
            if (this.active) {
                return Base.prototype.activate.apply(this, arguments);
            }
            this.render();
            return Base.prototype.activate.apply(this, arguments);
        },
        events: {
            'click a': function(e){
                this.model.report.entry.content.set('display.general.type', $(e.currentTarget).attr('data-type'));
                e.preventDefault();
            }
        },
        toggleActive: function() {
            (this.model.report.entry.content.get('display.general.type') == 'statistics') ?
                this.$el.addClass('active') :
                this.$el.removeClass('active');
        },
        render: function() {
            this.$el.html(this.compiledTemplate({
                _: _,
                searchJob: this.model.searchJob,
                i18n: i18n
            }));
            this.toggleActive();
            return this;
        },
        template:'\
            <a href="#" data-type="statistics">\
                <% if (searchJob.isNew() || !searchJob.entry.content.get("reportSearch")) { %>\
                    <%- _("Statistics").t() %>\
                <% } else { %>\
                    <%- _("Statistics").t() %> (<%= i18n.format_decimal(searchJob.resultCountSafe()) %>)\
                <% } %>\
            </a>\
        '
    });
});
