define(
    [
        'jquery',
        'underscore',
        'module',
        'keyboard/SearchModifier',
        'models/services/search/IntentionsParser',
        'views/Base',
        'views/search/results/statisticspane/StatisticsControls',
        'views/shared/results_table/ResultsTableMaster',
        'views/shared/JobDispatchState',
        'views/search/results/shared/NoStatistics',
        'uri/route'
    ],
    function($, _, module, KeyboardSearchModifier, IntentionsParser, Base, StatisticsControls, ResultsTable, JobDispatchState, NoStatistics, route){
        return Base.extend({
            moduleId: module.id,
            className: 'tab-pane',
            modifierKey: false,
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);

                this.keyboardSearchModifier = new KeyboardSearchModifier();
                this.model.intentionsParser = new IntentionsParser();

                //child views
                this.children.jobDispatchState = new JobDispatchState({
                    model: this.model.searchJob,
                    mode: this.model.searchJob.entry.content.get('isPreviewEnabled') ? 'results_preview' : 'results'
                });
                
                this.children.noStatistics = new NoStatistics({
                    model: {
                        report: this.model.report,
                        appLocal: this.model.appLocal,
                        application: this.model.application
                    }
                });
                
                this.children.statisticsControls = new StatisticsControls({
                    model: {
                        report: this.model.report,
                        timeline: this.model.timeline,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    }
                });
                
                this.children.resultsTable = new ResultsTable({
                    model: {
                        results: this.model.resultJsonRows,
                        metadata: this.model.report.entry.content,
                        format: this.model.report.entry.content
                    },
                    sortKeyAttr: 'display.statistics.sortColumn',
                    sortDirAttr: 'display.statistics.sortDirection',
                    countAttr: 'display.prefs.statistics.count',
                    offsetAttr: 'display.prefs.statistics.offset',
                    tableDockOffset: 36
                });
            },
            startListening: function() {
                this.listenTo(this.model.intentionsParser, 'change', function() {
                    var attrs = $.extend({}, {
                        'search': this.model.intentionsParser.get('fullSearch'),
                        'dispatch.earliest_time': this.model.intentionsParser.get('dispatch.earliest_time'),
                        'dispatch.latest_time': this.model.intentionsParser.get('dispatch.latest_time')
                    });
                    if(this.modifierKey == true) {
                       var routeString = route.search(
                                this.model.application.get('root'),
                                this.model.application.get('locale'),
                                this.model.application.get('app'),
                                {data: {
                                    earliest: this.model.intentionsParser.get('dispatch.earliest_time') ? this.model.intentionsParser.get('dispatch.earliest_time') : this.model.report.entry.content.get('dispatch.earliest_time'),
                                    latest: this.model.intentionsParser.get('dispatch.latest_time') ? this.model.intentionsParser.get('dispatch.latest_time') : this.model.report.entry.content.get('dispatch.latest_time'),
                                    q: this.model.intentionsParser.get('fullSearch')
                                }}
                       );
                       window.open(routeString,'_blank');
                    }
                    else {
                       this.model.report.entry.content.set(attrs);
                    }
                });
                
                this.listenTo(this.children.resultsTable, 'cellClick', function(clickMeta) {
                    var field, value;
                    this.model.intentionsParser.clear({silent: true});
                    this.modifierKey = clickMeta.modifierKey;
                    
                    if (clickMeta._span) {
                        this.model.intentionsParser.set({
                            'dispatch.earliest_time': clickMeta.value,
                            'dispatch.latest_time': JSON.stringify(parseFloat(clickMeta.value) + clickMeta._span)
                        }, {silent: true});
                        delete clickMeta.name;
                        delete clickMeta.value;
                        delete clickMeta.rowContext['row._time'];
                    }

                    if (this.model.report.entry.content.get('display.statistics.drilldown') === 'row') {
                        field = _(clickMeta.rowContext).keys().map(function(f) { return f.replace('row.', ''); });
                        value = _(clickMeta.rowContext).values();
                    } else if (this.model.report.entry.content.get('display.statistics.drilldown') === 'cell') {
                        if(clickMeta.name && clickMeta.name2){
                            field = [clickMeta.name, clickMeta.name2];
                            value = [clickMeta.value, clickMeta.value2];
                        } else if(clickMeta.name) {
                            field = clickMeta.name;
                            value = clickMeta.value;
                        } else {
                            field = clickMeta.name2;
                            value = clickMeta.value2;
                        }
                    }

                    this.model.intentionsParser.fetch({
                         data: $.extend(true, {}, {
                            q: this.model.report.entry.content.get('search'),
                            action: 'fieldvalue',
                            field: field || '',
                            value: value || '',
                            fieldMetaData: JSON.stringify(this.model.searchJob.entry.content.get('fieldMetadataResults')),
                            app: this.model.application.get('app'),
                            owner: this.model.application.get('owner')
                        })
                    });
                });
                
                this.listenTo(this.model.report.entry.content, 'change:display.visualizations.type', function() {
                    if (this.active) {
                        this.visibility();
                    }
                });
                this.listenTo(this.model.searchJob.entry.content, 'change:resultPreviewCount change:resultCount change:dispatchState change:isPreviewEnabled',  _.debounce(function() {
                    if (this.active) {
                        this.visibility();
                    }
                }, 0));
            },
            activate: function(options) {
                var clonedOptions = _.extend({}, (options || {}));
                delete clonedOptions.deep;
                this.ensureDeactivated({deep:true});
                Base.prototype.activate.call(this, clonedOptions);
                this.visibility();
                return this;
            },
            deactivate: function(options) {
                if (!this.active) {
                    return Base.prototype.deactivate.apply(this, arguments);
                }
                Base.prototype.deactivate.apply(this, arguments);
                this.model.intentionsParser.clear();
                return this;
            },
            visibility: function() {
                var resultCount = this.model.searchJob.entry.content.get('isPreviewEnabled') ? this.model.searchJob.entry.content.get('resultPreviewCount') : this.model.searchJob.entry.content.get('resultCount');
                this.children.jobDispatchState.options.mode = this.model.searchJob.entry.content.get('isPreviewEnabled') ? 'results_preview' : 'results';

                if (!this.model.searchJob.isReportSearch()) {
                    this.children.jobDispatchState.deactivate({deep: true}).$el.hide();
                    this.children.noStatistics.activate({deep: true}).$el.show();
                    this.children.statisticsControls.deactivate({deep: true}).$el.hide();
                    this.children.resultsTable.deactivate({deep: true}).$el.hide();
                    return;
                }
                if (resultCount === 0) {
                    this.children.jobDispatchState.activate({deep: true}).$el.show();
                    this.children.noStatistics.deactivate({deep: true}).$el.hide();
                    this.children.statisticsControls.activate({deep: true}).$el.show();
                    this.children.resultsTable.deactivate({deep: true}).$el.hide();
                } else {
                    this.children.jobDispatchState.deactivate({deep: true}).$el.hide();
                    this.children.noStatistics.deactivate({deep: true}).$el.hide();
                    this.children.statisticsControls.activate({deep: true}).$el.show();
                    this.children.resultsTable.$el.show();
                    this.children.resultsTable.activate({deep: true});
                }
            },
            render: function() {
                if (!this.el.innerHTML) {
                    this.children.noStatistics.render().appendTo(this.$el);
                    this.children.statisticsControls.render().appendTo(this.$el);
                    this.children.jobDispatchState.render().appendTo(this.$el);
                    this.children.resultsTable.render().appendTo(this.$el);
                }
                return this;
            }
    });
});
