define(
    [
        'jquery',
        'underscore',
        'module',
        'views/Base'
    ],
    function($, _, module, BaseView) {
        return BaseView.extend({
            moduleId: module.id,
            className: "json-tree",
            /**
             * @param {Object} options {
             *     json: <json stringified object>
             *     isValidJSON: <a flag allowing opt-out of safe set json routine>
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                
                var defaults = {
                        isValidJSON: false,
                        json: '{}'
                };
                this.options = $.extend(true, defaults, this.options);

                if(this.options.isValidJSON) {
                    this._json = this.options.json;
                } else {
                    this.setJSON(this.options.json);
                }
            },
            setJSON: function(json) {
                var parsed;

                if(typeof json !== 'string') {
                    json = JSON.stringify(json);
                } 

                try{
                    parsed = JSON.parse(json);
                } catch(e) {}
                
                this._json = parsed;
            }, 
            isValidJSON: function() {
                return !!this._json;
            },
            events: {
                'click .jsexpands': function(e) {
                    var $target = $(e.currentTarget);
                    this.trigger('interaction');
                    this.interacted = true;
                    $target.removeClass('jsexpands').addClass('jscollapse').text('[-]').next().show();
                    e.preventDefault();
                },
                'click .jscollapse': function(e) {
                    var $target = $(e.currentTarget);
                    this.trigger('interaction');
                    this.interacted = true;
                    $target.removeClass('jscollapse').addClass('jsexpands').text('[+]').next().hide();
                    e.preventDefault();
                }
            },
            render: function() {
                if(!this.interacted) {
                    this.$el.html(this.compiledTemplate({ 
                        obj: this._json,
                        indent: 0,
                        template: this.compiledTemplate,
                        _: _
                    }));
 
                }
                return this;
            },
            template: '\
                <% var indent = indent + 2; %>\
                <% var level  = indent/2; %>\
                <% var type   = (obj === null) ? "null": typeof obj; %>\
                \
                <% if(!_.isObject(obj)) { %>\
                    <span class="t <%- type %>"><%- obj || "null" %></span>\
                <% } else { %>\
                    <% var isObj = !(obj instanceof Array) %>\
                    <% var list  = (isObj) ? _.keys(obj).sort() : obj.sort(); %>\
                    <span><%if(isObj) { %>{<% } else { %>[<% } %></span>\
                    \
                    <% if (list.length > 0) { %>\
                        <% if(!(level < 2)) { %>\
                            <a class="jsexpands">[+]</a>\
                        <% } else { %>\
                            <a class="jscollapse">[-]</a>\
                        <% } %>\
                    <% } %>\
                    \
                    <% if (level != 1) { %>\
                        <span style="display: none;">\
                    <% } else { %>\
                        <span>\
                    <% } %>\
                    \
                    <% var i = 0; %>\
                    <% for(; i<list.length; i++){ %>\
                        <br><% for(var j=0; j<indent; j++){ %>&nbsp;<% } %>\
                        <% if(isObj) { %>\
                            <span class="key level-<%-level%>">\
                                <span class="key-name"><%-list[i]%></span>:\
                                <%= template({ obj:obj[list[i]], indent: indent, template: template, _:_}) %>\
                            </span>\
                        <% } else { %>\
                            <%= template({ obj:list[i], indent: indent, template: template, _:_}) %>\
                        <% } %>\
                    <% } %>\
                    </span><br>\
                    <span>\
                        <% for(var j=0; j<indent-2; j++){ %>&nbsp;<% } %>\
                        <% if(isObj){ %><span>}</span><% } else { %><span>]</span><%}%>\
                    </span>\
                \
                <% } %>\
                \
            '
        });
    }
 );
