define(
    [
        'module',
        'views/Base',
        'views/shared/splunkbar/Master',
        'views/shared/appbar/Master',
        'views/shared/footer/Master'
    ],
    function(
        module,
        BaseView,
        SplunkBarView,
        AppBarView,
        FooterView
    ) {
        return BaseView.extend({
            moduleId: module.id,
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                if (this.options.splunkBar) {
                    this.children.splunkBar = SplunkBarView.create({
                        model: {
                            application: this.model.application,
                            appNav: this.model.appNav,
                            appLocal: this.model.appLocal,
                            user: this.model.user,
                            serverInfo: this.model.serverInfo,
                            config: this.model.config
                        },
                        collection: this.collection
                    });
                }
                if (this.model.appNav) {
                    this.children.appBar = AppBarView.create({
                        section: this.options.section || '',
                        model: {
                            application: this.model.application,
                            appNav: this.model.appNav
                        },
                        collection: this.collection
                    });
                }
                if (this.options.footer) {
                    this.children.footer = FooterView.create({
                        model: this.model,
                        collection: this.collection
                    });
                }
            },
            render: function() {
                var html = this.compiledTemplate({loadingMessage: this.options.loadingMessage || ''});
                this.$el.html(html);
                if (this.options.splunkBar) {
                    this.$('header').append(this.children.splunkBar.render().el);
                }
                if (this.model.appNav) {
                    this.$('header').append(this.children.appBar.el);
                }
                if (this.options.footer) {
                    this.$('footer').append(this.children.footer.render().el);
                }
                return this;
            },
            template: '\
                <a class="navSkip" href="#navSkip" tabIndex="1">Screen reader users, click here to skip the navigation bar</a>\
                <header role="banner"></header>\
                <a id="navSkip"></a> \
                <div class="main-section-body" role="main">\
                    <% if (loadingMessage) { %>\
                        <div class="loading-message"><%- loadingMessage %></div>\
                    <% } %>\
                </div>\
                <footer role="contentinfo"></footer>\
            '
        });
    }
);
