define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/delegates/PopdownDialog'
    ],
    function(_, module, Base, PopdownDialogDelegate) {
        return Base.extend({
            moduleId: module.id,
            className: 'popdown-dialog',
            initialize: function(options) {
                options = options || {};
                var defaults = {
                    direction:'auto',
                    adjustPosition: true
                };
                _.defaults(options, defaults);
                
                Base.prototype.initialize.apply(this, arguments);
                this.children.popdownDialogDelegate = new PopdownDialogDelegate({
                    el: this.el,
                    ignoreClasses: this.options.ignoreClasses, 
                    adjustPosition: this.options.adjustPosition,
                    show: this.options.show,
                    mode: this.options.mode, // "menu" or "dialog"
                    direction: this.options.direction,
                    arrow: this.options.arrow,
                    minMargin: this.options.minMargin,
                    allowPopUp: this.options.allowPopUp,
                    scrollContainer: this.options.scrollContainer
                });
                this.activate();
            },
            startListening: function() {
                this.listenTo(this.children.popdownDialogDelegate, 'all', function() {
                    this.trigger.apply(this, arguments);
                });
                this.listenTo(this, 'shown', function() {
                    this.shown = true;
                    this.children.popdownDialogDelegate.$el.find('a').first().focus();
                });
                this.listenTo(this, 'hidden', function() {
                    this.shown = false;
                    if (this.options.onHiddenRemove) {
                        this.remove();
                    }
                });
                this.listenTo(this, 'focus-on-activator', function() {
                    if (this.$toggle) {
                        this.$toggle.focus();
                    }
                });
            },
            toggle: function () {
                return this.children.popdownDialogDelegate.toggle();
            },
            show: function ($toggle) {
                this.children.popdownDialogDelegate.show($toggle);
                this.$toggle = $toggle;
            },
            hide: function () {
                this.children.popdownDialogDelegate.hide();
            },
            render: function() {
                this.el.innerHTML = this.template;
                return this;
            },
            remove: function() {
                if(this.shown) {
                    this.hide();
                }

                Base.prototype.remove.apply(this, arguments);

            },
            template: '\
                <div class="arrow"></div>\
                <div class="popdown-dialog-body popdown-dialog-padded"></div>\
            ',
            template_menu: '\
                <div class="arrow"></div>\
            '
        });
    }
);
