define(
    [
        'underscore',
        'module',
        'views/Base',
        'views/shared/Modal',
        'views/shared/FlashMessages',
        'uri/route'
     ],
     function(_, module, Base, Modal, FlashMessages, route){
        return Base.extend({
            moduleId: module.id,
             /**
             * @param {Object} options {
             *     model: {
             *         alert: <models.Report>,
             *         application: <models.Application>
             *     }
             * }
             */
            initialize: function() {
                Base.prototype.initialize.apply(this, arguments);
                this.children.flashMessage = new FlashMessages({ model: this.model.alert });
                this.model.alert.on('change:id', function() {
                    this.setText();
                }, this);
            },
            setText: function() {
                if (this.model.alert.entry.acl.get('can_change_perms')) {
                    this.$('span.alert-success-message').text(_('You can view your alert, change additional settings, or continue editing it.').t());
                    this.$('.can-change-perms').show();
                } else {
                    this.$('span.alert-success-message').text(_('You can view your alert, or continue editing it.').t());
                    this.$('.can-change-perms').hide();
                }
            },
            render: function() {
                var routeToAlert = route.alert(this.model.application.get('root'),
                                     this.model.application.get('locale'),
                                     this.model.application.get("app"),
                                     {data: {s: this.model.alert.id}});
                
                this.$el.html(Modal.TEMPLATE);

                this.$(Modal.HEADER_TITLE_SELECTOR).html(_("Alert has been saved").t());
                this.$(Modal.BODY_SELECTOR).html(this.compiledTemplate({
                    _: _,
                    routeToPermissions : route.alert(this.model.application.get('root'),
                                           this.model.application.get('locale'),
                                           this.model.application.get("app"),
                                           {data: {s: this.model.alert.id, dialog: 'permissions'}})
                }));
                this.children.flashMessage.render().prependTo(this.$(Modal.BODY_SELECTOR));
                this.setText();

                this.$(Modal.FOOTER_SELECTOR).append(Modal.BUTTON_CONTINUE);
                this.$(Modal.FOOTER_SELECTOR).append('<a href="' + routeToAlert + '" class="btn btn-primary modal-btn-primary">' + _("View Alert").t() + '</a>');

                return this;
            },
            template: '\
                <p>\
                    <span class="alert-success-message"></span>\
                </p>\
                <p class="can-change-perms">\
                    <%- _("Additional Settings:").t() %>\
                    <ul class="can-change-perms">\
                        <li><a href="<%- routeToPermissions %>"><%- _("Permissions").t() %></a></li>\
                    <ul>\
                </p>\
            '
        });
    }
);