define([
            'underscore',
            'module',
            'views/shared/controls/Control',
            'splunk.util',
            'select2/select2'
        ],
        function(
            _,
            module,
            Control,
            splunkUtils
            /* remaining modules do not export */
        ) {

    var DELIMITER = '::::';

    return Control.extend({

        moduleId: module.id,

        events: {

            'change input': function(e) {
                var values = e.val || [];
                this.setValue(splunkUtils.fieldListToString(values), false);
            }

        },

        /**
         * @constructor
         * @param {Object} options {
         *     {String, required} modelAttribute The attribute on the model to observe and update on selection
         *     {Model, required} model The model to operate on
         *     {String, optional} placeholder The placeholder text for an empty input
         *     {Array<String>, optional} autoCompleteFields A list of fields to use for auto-complete
         *     {String, optional} inputClassName A class name to apply to the input element
         * }
         */

        render: function() {
            if(this.el.innerHTML) {
                return;
            }
            this.$el.html(this.compiledTemplate({ options: this.options }));
            var $input = this.$('input');
            $input.select2({
                placeholder: this.options.placeholder,
                tags: this.options.autoCompleteFields || [],
                formatNoMatches: function() { return '&nbsp;'; },
                dropdownCssClass: 'empty-results-allowed',
                separator: DELIMITER,
                // SPL-77050, this needs to be false for use inside popdowns/modals
                openOnEnter: false,
                // SPL-77130, the default initSelection has a bizarre bug in minify_js mode
                initSelection: function($element, callback) {
                    var val = $element.val();
                    if(!val) {
                        callback([]);
                        return;
                    }
                    var data = _(val.split(DELIMITER)).map(function(str) {
                        return { id: str, text: str };
                    });
                    callback(data);
                }
            })
            .select2('val', splunkUtils.stringToFieldList(this._value || ''))
            // SPL-81507, similar to comment above about bugginess in minify_js mode, select2's string comparisons
            // are not working, so we roll our own way to make sure items are removed properly
            .on('removed', function(e) {
                var removedChoice = e.val.toString(),
                    val = $input.val();

                val = _(val).isString() ? val.split(DELIMITER) : val;
                var updatedVal = _(val).without(removedChoice);
                $input.select2('val', updatedVal);
            })
            // SPL-80073, similar to comment above about bugginess in minify_js mode, we roll our own way to make sure
            // items can't be selected twice
            .on('open', function() {
                var val = $input.val(),
                    $results = $input.data('select2').results;

                val = _(val).isString() ? val.split(DELIMITER) : val;
                $results.find('.select2-result').each(function() {
                    var $choice = $(this),
                        id = $choice.data('select2-data').id.toString();

                    if(_(val).contains(id)) {
                        $choice.addClass('select2-selected');
                    }
                });
            });
            return this;
        },

        remove: function() {
            this.$('input').select2('close').select2('destroy');
            return Control.prototype.remove.apply(this, arguments);
        },

        template: '\
            <input class="<%= options.inputClassName %>" />\
        '

    });

});