define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/eventsviewer/list/body/row/SelectedFields',
        'views/shared/eventsviewer/shared/EventFields',
        'views/shared/eventsviewer/shared/RawField',
        'splunk.util'
    ],
    function(
        $,
        _,
        Backbone,
        module,
        BaseView,
        SelectedFieldsView,
        EventFieldsView,
        RawField,
        splunkUtil
    ){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tr',
            /**
             * @param {Object} options {
             *      model: {
             *         event: <models.services.search.job.ResultsV2.results[i]>,
             *         result: <models.services.search.job.ResultsV2>,
             *         state: <models.Base>,
             *         summary: <models.services.searchjob.SummaryV2>,
             *         report: <models.services.SavedSearch>,
             *         searchJob: <models.Job>,
             *         application: <models.Application>
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>,
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *         workflowActions: <collections.services.data.ui.WorkflowActions>
             *     },
             *     selectableFields: true|false
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                
                this.interaction  = 'i' + this.options.idx;
                this.rowExpanded  = 'r' + this.options.idx;
                this.showAllLines = 's' + this.options.idx;
                
                this.model.state.unset(this.showAllLines);

                this.model.renderer = this.collection.eventRenderers.getRenderer(this.model.event.get('eventtype'));

                this.children.raw = new RawField({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        report: this.model.report,
                        searchJob: this.model.searchJob,
                        application: this.model.application
                    },
                    collection: {
                        eventRenderers: this.collection.eventRenderers
                    },
                    idx: this.options.idx
                });

                this.children.selectedFields = new SelectedFieldsView({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        summary: this.model.summary,
                        application: this.model.application,
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        workflowActions: this.collection.workflowActions
                    },
                    rowExpanded: this.rowExpanded,
                    selectableFields: this.options.selectableFields
                });
            },
            startListening: function() {
                this.listenTo(this.model.state, this.interaction, function() {
                    if(!this.isExpanded()) {
                        this.expand();
                    }
                });
                
                /*
                 * Our bus for communication from our grandparent regarding clicks on the
                 * modalization mask.
                 */
                this.listenTo(this.model.state, 'change:' + this.rowExpanded, function(model, value, options){
                    if(!value) {
                        this.collapseState();
                    }
                });
                
                this.listenTo(this.model.state, 'change:' + this.showAllLines, function(model, value, options){
                    (!this.isExpanded() && this.options.allowRowExpand) ? this.expand(): this.eventFetch();
                });
                
                //on change of the search string we should unmodalize
                this.listenTo(this.model.state, 'intentions-fetch', this.collapseState);
                
                this.listenTo(this.model.report.entry.content, 'change:display.events.type', this.manageListRawState);
            },
            manageListRawState: function() {
                if (this.isList()) {
                    this.$('td._time').show();
                    if(!this.children.selectedFields.active) {
                        this.children.selectedFields.activate().$el.show(); 
                    } else {
                        this.children.selectedFields.$el.show(); 
                    }
                } else {
                    this.children.selectedFields.deactivate().$el.hide(); 
                    this.$('td._time').hide();
                }
            },
            events: {
                'click td.expands': function(e) {
                    this.expand();
                    e.preventDefault();
                },
                'click .formated-time': function(e) {
                    e.preventDefault();
                    this.drilldown($(e.currentTarget), e);
                },
                'keyup ._time': function(e) {
                    e.preventDefault();
                    if(e.which === 13) {
                        this.drilldown($(e.currentTarget).find('span.formated-time'), e);
                    }
                }
            },
            drilldown: function($target, e) {
                //TODO: this looks exactly like table/body/PrimaryRow's drilldown method. They should come from a base class most likely.
                var data = $target.data(), timeIso, epoch;
                if (data.timeIso) {
                    timeIso = data.timeIso;
                    epoch = splunkUtil.getEpochTimeFromISO(timeIso);
                    this.model.state.trigger('drilldown', {
                        noFetch: true, 
                        data: {
                            'dispatch.earliest_time': epoch,
                            'dispatch.latest_time': '' + (parseFloat(epoch) + 1)
                        },
                        event: e,
                        _time: timeIso,
                        idx: this.options.idx
                    });                    
                }                
            },
            eventFieldsFactory: function() {
                this.children.eventFields = new EventFieldsView({
                    model: {
                        event: this.model.event,
                        state: this.model.state,
                        result: this.model.result,
                        summary: this.model.summary,
                        application: this.model.application,
                        report: this.model.report,
                        searchJob: this.model.searchJob
                    },
                    collection: {
                        selectedFields: this.collection.selectedFields,
                        workflowActions: this.collection.workflowActions
                    },
                    selectableFields: this.options.selectableFields,
                    allowRowExpand: this.options.allowRowExpand,
                    idx: this.options.idx,
                    swappingKey: this.swappingKey,
                    showAllLines: this.showAllLines
                });
                //remove and put into another function
                this.children.eventFields.render().insertAfter(this.$('.event').find(this.children.raw.el));
            },
            expand: function(options) {
                if (this.options.allowRowExpand) {
                    if(!this.children.eventFields) {
                        this.eventFieldsFactory();
                    }
                    (this.isExpanded()) ? this.collapseState(): this.expandState();
                }
            },
            eventFetch: function(showAll) {
                this.model.event.id = this.model.searchJob.entry.links.get('events');
                this.model.event.fetch({
                    data: $.extend(true, this.model.application.toJSON(), {
                        isRt: this.model.searchJob.isRealtime(),
                        search: this.model.report.getSortingSearch(),
                        earliest_time: this.model.report.entry.content.get('display.events.timelineEarliestTime'),
                        latest_time: this.model.report.entry.content.get('display.events.timelineLatestTime'),
                        segmentation:  this.model.report.entry.content.get('display.events.list.drilldown'),
                        max_lines: this.model.state.get(this.showAllLines) ? 0: this.model.report.getNearestMaxlines(),
                        eventsOffset: +this.model.report.entry.content.get('display.prefs.events.offset') || 0,
                        oid: this.model.application.get('oid')
                    })     
                });
            },
            isExpanded: function() {
                return this.model.state.get(this.rowExpanded);
            },
            expandState: function() {
                this.eventFetch();
                this.model.state.set(this.rowExpanded, true);
                this.children.eventFields && this.children.eventFields.activate().$el.show();
                this.isList() && this.children.selectedFields.deactivate().$el.hide();
                this.toggleArrow(true);
            },
            collapseState: function() {
                this.model.state.set(this.rowExpanded, false);                
                this.children.eventFields && this.children.eventFields.deactivate().$el.hide();
                this.isList() && this.children.selectedFields.activate().$el.show();
                this.toggleArrow(false);
            },
            isList: function() {
                return (this.model.report.entry.content.get('display.events.type') === 'list');
            },
            toggleArrow: function(open) {
                var $arrow =  this.$('td.expands > a > i').removeClass('icon-triangle-right-small icon-triangle-down-small');
                $arrow.addClass((open) ? 'icon-triangle-down-small':  'icon-triangle-right-small');
            },
            render: function() {
               this.$el.html(this.compiledTemplate({
                    $: $,
                    event: this.model.event,
                    lineNum: this.options.lineNum,
                    application: this.model.application, //ghetto and inconsistent 
                    expanded: this.isExpanded(),
                    isList: this.isList(),
                    formattedTime: this.model.event.formattedTime(),
                    colorClass: (this.model.renderer) ? this.model.renderer.entry.content.get('css_class'): '',
                    allowRowExpand: this.options.allowRowExpand //misuse by data model
                }));
               
                this.children.raw.render().appendTo(this.$('.event'));
                this.children.selectedFields.render().appendTo(this.$('.event'));

                if(!this.isList()) {
                    this.children.selectedFields.deactivate().$el.hide();
                }

                return this;
            },
            template: '\
                <% if (allowRowExpand) { %>\
                <td class="expands <%- colorClass %>">\
                    <a href="#"><i class="icon-triangle-<%- expanded ? "down" : "right" %>-small"></i></a>\
                </td>\
                <% } %>\
                <td class="line-num"><span><%- lineNum %></span></td>\
                <td class="_time" <% if (!isList) { %> style="display:none" <%}%> tabindex="0">\
                    <span class="formated-time" data-time-iso="<%- event.get("_time") %>">\
                    <% if(application.get("locale").indexOf("en") > -1){ %>\
                         <span><%- $.trim(formattedTime.slice(0, formattedTime.indexOf(" "))) %></span>\
                         <br>\
                         <span><%- $.trim(formattedTime.slice(formattedTime.indexOf(" "))) %></span>\
                    <% } else { %>\
                         <%- formattedTime %>\
                    <% } %>\
                    </span>\
                </td>\
                <td class="event"></td>\
            '
        });
    }
);
