define(
    [
        'jquery',
        'underscore',
        'module',
        'keyboard/SearchModifier',
        'views/Base',
        'splunk.util'
    ],
    function($, _, module, KeyboardSearchModifier, BaseView, splunkUtil){
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'tr',
            /**
             * @param {Object} options {
             *     model: { 
             *         event: <models.services.search.job.ResultsV2.results[i]>,
             *         report: <models.services.SavedSearch>,
             *         state: <models.Base>
             *     },
             *     collection: {
             *         selectedFields: <collections.SelectedFields>
             *         eventRenderers: <collections.services.configs.EventRenderers>,
             *     }
             *     
             */
            initialize: function(){
                BaseView.prototype.initialize.apply(this, arguments);
                this.$el.addClass((!!(this.options.idx%2))?'even':'odd');
                
                this.keyboardSearchModifier = new KeyboardSearchModifier();

                this.model.renderer = this.collection.eventRenderers.getRenderer(this.model.event.get('eventtype'));
                
                //suite of namespaced keys
                this.rowExpanded = 'r' + this.options.idx;
                this.interaction = 'i' + this.options.idx;
                this.showAllLines = 's' + this.options.idx;
            },
            startListening: function() {
                /*
                 * Our bus for communication from our grandparent regarding clicks on the
                 * modalization mask.
                 */
                this.listenTo(this.model.state, 'change:' + this.rowExpanded, function(model, value, options) {
                    if (!value) {
                        this.collapseState();
                    }
                    this.$('.expands').attr('rowspan', (value) ? 2: 1);
                });
                 
                 /*
                  * Columns in the table are dynamic based on the users 
                  * field selection.
                  */
                this.listenTo(this.collection.selectedFields, 'reset add remove', this.render);

                this.listenTo(this.model.report.entry.content, 'change:display.events.table.drilldown', this.render);

                this.listenTo(this.model.report.entry.content, 'change:display.prefs.events.offset', this.collapseState);
                this.listenTo(this.model.report.entry.content, 'change:display.events.list.wrap', function() {
                    var wrap = splunkUtil.normalizeBoolean(this.model.report.entry.content.get("display.events.list.wrap")),
                        $cells = this.$('a.field-val').parents('td');
                   (wrap) ? $cells.removeClass('no-wrap'):  $cells.addClass('no-wrap');
                });
            },
            events: {
                'click td.expands': function(e) {
                    this.expand();
                    e.preventDefault();
                },
                'click ._time-drilldown > a': function(e) {
                    e.preventDefault(); // handled by the cell.
                },
                'click ._time-drilldown': function(e) {
                    e.preventDefault();
                    if(splunkUtil.normalizeBoolean(this.model.report.entry.content.get('display.events.table.drilldown'))){
                        this.drilldown($(e.currentTarget).find('a'), e);
                    }
                },
                'click .one-value-drilldown > a.field-val': function(e) {
                    e.preventDefault(); // handled by the cell.
                },
                'click .one-value-drilldown': function(e) {
                    e.preventDefault();
                    this.drilldown($(e.currentTarget).find('a'), e);
                },
                'click .multi-value-drilldown > a.field-val': function(e) {
                    e.preventDefault();
                    this.drilldown($(e.currentTarget), e);
                },
                'keypress td': function(e) {
                    e.preventDefault();
                    if(e.which === 13) {
                        this.drilldown($(e.currentTarget).find('a'), e);
                    }
                } 
            },
            expand: function(options) {
                (this.isExpanded()) ? this.collapseState(): this.expandState();
            },
            isExpanded: function() {
                return this.model.state.get(this.rowExpanded);
            },
            eventFetch: function() {
                this.model.event.id = this.model.searchJob.entry.links.get('events');
                this.model.event.fetch({
                    data: $.extend(true, this.model.application.toJSON(), {
                        isRt: this.model.searchJob.isRealtime(),
                        search: this.model.report.getSortingSearch(),
                        earliest_time: this.model.report.entry.content.get('display.events.timelineEarliestTime'),
                        latest_time: this.model.report.entry.content.get('display.events.timelineLatestTime'),
                        segmentation:  this.model.report.entry.content.get('display.events.list.drilldown'),
                        max_lines: this.model.state.get(this.showAllLines) ? 0: this.model.report.getNearestMaxlines(),
                        eventsOffset: +this.model.report.entry.content.get('display.prefs.events.offset') || 0
                    })     
                });
            },
            debouncedRemove: function() {
                this.model.state.set(this.pendingRemove, true);
                BaseView.prototype.debouncedRemove.apply(this, arguments);
                return this;
            },
            drilldown: function($target, e) {
                //TODO: this looks exactly like list/body/row/Master's drilldown method. They should come from a base class most likely.
                var data = $target.data(), timeIso, epoch;
                if (data.timeIso) {
                    timeIso = data.timeIso;
                    epoch = splunkUtil.getEpochTimeFromISO(timeIso);
                    this.model.state.trigger('drilldown', {
                        noFetch: true, 
                        data: {
                            'dispatch.earliest_time': epoch,
                            'dispatch.latest_time': '' + (parseFloat(epoch) + 1)
                        },
                        event: e,
                        _time: timeIso,
                        idx: this.options.idx
                    });                    
                } else {
                    this.model.state.trigger('drilldown', {
                        data: {
                             q: this.model.report.entry.content.get('search'),
                             negate: this.keyboardSearchModifier.isNegation(e),
                             action: 'fieldvalue', 
                             field: $target.data().name,
                             value: $.trim($target.text()),
                             app: this.model.application.get('app'),
                             owner: this.model.application.get('owner')
                         },
                         event: e,
                         idx: this.options.idx
                    });                    
                }
            },
            expandState: function() {
                this.eventFetch();
                this.model.state.set(this.rowExpanded, true);
                this.toggleArrow(true);
            },
            collapseState: function() {
                this.model.state.set(this.rowExpanded, false);                
                this.toggleArrow(false);
            },
            toggleArrow: function(open) {
                var $arrow =  this.$('td.expands > a > i').removeClass('icon-triangle-right-small icon-triangle-down-small');
                $arrow.addClass((open) ? 'icon-triangle-down-small':  'icon-triangle-right-small');
            },
            render: function() {
                var root = this.el;
                //rows are read only (innerHTML) for ie
                this.$el.find('> td').each(function(key, element) {
                    root.removeChild(element);
                });
                this.$el.append(this.compiledTemplate({
                    $: $,
                    _:_,
                    event: this.model.event,
                    lineNum: this.options.lineNum,
                    expanded: this.model.state.get(this.rowExpanded),
                    drilldown: splunkUtil.normalizeBoolean(this.model.report.entry.content.get('display.events.table.drilldown')),
                    application: this.model.application,
                    selectedFields: this.collection.selectedFields,
                    colorClass: (this.model.renderer) ? this.model.renderer.entry.content.get('css_class'): '',
                    formattedTime: this.model.event.formattedTime(),
                    allowRowExpand: this.options.allowRowExpand,
                    wrap: splunkUtil.normalizeBoolean(this.model.report.entry.content.get('display.events.table.wrap'))
                }));
                return this;
            },
            template: '\
                <% if (allowRowExpand) { %>\
                    <td <%if(expanded) {%>rowspan=2<%}%> class="expands <%- colorClass %>">\
                        <a href="#"><i class="icon-triangle-<%- expanded ? "down" : "right" %>-small"></i></a>\
                    </td>\
                <% } %>\
                <td class="line-num"><span><%- lineNum %></span></td>\
                <td class="_time <%= drilldown ? "_time-drilldown" : "" %>" tabindex="0">\
                    <% if(drilldown) { %>\
                        <a data-time-iso="<%- event.get("_time") %>">\
                    <% } else { %>\
                        <span data-time-iso="<%- event.get("_time") %>">\
                    <% } %>\
                        <% if(application.get("locale").indexOf("en") > -1){ %>\
                             <span><%- $.trim(formattedTime.slice(0, formattedTime.indexOf(" "))) %></span>\
                             <br>\
                             <span><%- $.trim(formattedTime.slice(formattedTime.indexOf(" "))) %></span>\
                        <% } else { %>\
                             <%- formattedTime %>\
                        <% } %>\
                    <% if(drilldown) { %>  </a>  <% } else { %> </span> <% } %>\
                </td>\
                <% selectedFields.each(function(model) { %>\
                    <% var fields = event.get(model.get("name")); %>\
                    <% if (drilldown) { %>\
                        <td class="<% if(!wrap) { %>no-wrap<% } %> <%= drilldown && fields && fields.length > 1 ? "multi-value-drilldown" : ""  %> <%= drilldown && fields && fields.length == 1 ? "one-value-drilldown" : ""  %>"  tabindex="0"><% _(fields).each(function(field) { %><a class="field-val" data-name="<%- model.get("name") %>"><%- field %></a><% }) %></td>\
                    <% } else { %>\
                        <td class="<% if(!wrap) { %>no-wrap<% } %>"  tabindex="0"><% _(fields).each(function(field) { %><span class="field-val" data-name="<%- model.get("name") %>"><%- field %></span><% }) %></td>\
                    <% } %>\
                <% }) %>\
            '
        });
    }
);  
