define(['underscore','module', 'views/Base', 'bootstrap.tooltip'], function(_, module, Base /* bootstrap tooltip */) {
    return Base.extend({
        moduleId: module.id,
        className: 'progress-bar',
        tagName: 'div',
        initialize: function(){
            Base.prototype.initialize.apply(this, arguments);
            this.activate({skipRender: true});
        },
        startListening: function() {
            this.listenTo(this.model, 'sync', this.render);  
        },
        activate: function(options) {
            options = options || {};            
            
            if (this.active) {
                return Base.prototype.activate.apply(this, arguments);
            }
            
            if (!options.skipRender) {
                this.render();
            }
            
            return Base.prototype.activate.apply(this, arguments);
        },
        render: function() {
            if(!this.el.innerHTML) {
                this.$el.html(this.compiledTemplate({
                    _: _
                }));
                this.$('.progress-label-percentage').tooltip({animation:false, title:_('Percentage of the time range scanned.').t()});
            }
            
            var isRealTimeSearch = this.model.entry.content.get("isRealTimeSearch");
            
            if (this.model.isRunning() && !isRealTimeSearch){
                this.$el.show().children('span[class!="progress-label-percentage"]').hide();
                var percentage = (this.model.entry.content.get('doneProgress') * 100.0).toFixed(1);
                this.$('.progress-animation').show().css('width', percentage + '%');
                this.$('.progress-label-percentage').text(percentage + _('%').t()).show();
            } else {
                this.$el.show().children().hide();
                if (this.model.entry.content.get("isDone")) {
                    this.$('.progress-label-complete').show();
                } else if (this.model.isFinalizing()) {
                    this.$('.progress-label-finalizing').show();
                    this.$('.progress-animation-wrapper').show().css('width', '');
                } else if (this.model.entry.content.get("isPaused")) {
                    this.$('.progress-label-paused').show();
                } else if (isRealTimeSearch) {
                    this.$('.progress-animation-wrapper').show().css('width', '');
                    this.$('.progress-label-realtime').show();
                } else if (this.model.isQueued()) {
                    this.$('.progress-animation-wrapper').show().css('width', '');
                    this.$('.progress-label-queued').show();
                } else if (this.model.isParsing()) {
                    this.$('.progress-animation-wrapper').show().css('width', '');
                    this.$('.progress-label-parsing').show();
                } else {
                    this.$el.hide();
                }
            }

            return this;
        },
        template: '\
                <div class="progress-animation"></div>\
                <span class="progress-label progress-label-complete"><%- _("Complete").t() %></span>\
                <span class="progress-label progress-label-finalizing"><%- _("Finalizing...").t() %></span>\
                <span class="progress-label progress-label-paused"><%- _("Paused").t() %></span>\
                <span class="progress-label progress-label-realtime"><%- _("Real-time").t() %></span>\
        		<span class="progress-label progress-label-queued"><%- _("Queued").t() %></span>\
        		<span class="progress-label progress-label-parsing"><%- _("Parsing").t() %></span>\
                <span class="progress-label progress-label-percentage"><%- _("0%").t() %></span>\
        '
    });
});
