define(
    [
        'backbone',
        'module',
        'models/search/Dashboard',
        'views/shared/Modal',
        'views/shared/reportcontrols/dialogs/createreport/Create',
        'views/shared/reportcontrols/dialogs/createreport/Success',
        'views/shared/reportcontrols/dialogs/dashboardpanel/Create',
        'views/shared/reportcontrols/dialogs/dashboardpanel/Success',
        'util/splunkd_utils'
    ],
    function(Backbone, module, DashboardModel, Modal, Create, Success, CreateDashboardPanel, SuccessDashboardPanel, splunkd_utils){
            return Modal.extend({
                /**
                 * @param {Object} options {
                 *  model:{
                 *      report: <models.services.SavedSearch>,
                 *      application: <models.Application>,
                 *      searchJob: <models.services.search.Job> (optional),
                 *      user: <models.service.admin.user>
                 *  },
                 *  chooseVisualizationType {Boolean} whether to offer the user a choice of visualization type
                 *                                    defaults to true
                 * 
                 */
                moduleId: module.id,
                initialize: function() {
                    Modal.prototype.initialize.apply(this, arguments);
                    var chooseVisualizationType = this.options.hasOwnProperty('chooseVisualizationType') ?
                            this.options.chooseVisualizationType : true;

                    this.model.inmem = this.model.report.clone();

                    //cleanup of the report (no matter what the type)
                    delete this.model.inmem.id;
                    this.model.inmem.stripAlertAttributes();
                    this.model.inmem.entry.content.set({
                        'request.ui_dispatch_app': this.model.application.get('app'),
                        'request.ui_dispatch_view': this.model.application.get('page'),
                        // for backwards compatibility
                        'action.email.useNSSubject': '1',
                        //new reports should not be accelerated
                        'auto_summarize': '0'
                    });

                    this.model.dashboardToSave = new DashboardModel();
                    this.model.dasboardInmem = new Backbone.Model({
                        dashPerm: "private",
                        panelContent: "table",
                        dashCreateType: "new",
                        panelInline: true
                    });

                    this.children.create = new Create({
                        model: {
                            searchJob: this.model.searchJob,
                            application: this.model.application,
                            inmem: this.model.inmem
                        },
                        chooseVisualizationType: chooseVisualizationType
                    });

                    this.children.success = new Success({
                        model: {
                            application: this.model.application,
                            inmem: this.model.inmem,
                            searchJob: this.model.searchJob,
                            user: this.model.user
                        }
                    });

                    this.children.createDashboardPanel = new CreateDashboardPanel({
                        model: {
                            report: this.model.inmem,
                            application: this.model.application,
                            searchJob: this.model.searchJob,
                            dashboardToSave: this.model.dashboardToSave,
                            inmem: this.model.dasboardInmem
                        },
                        chooseVisualizationType: chooseVisualizationType
                    });

                    this.children.successDashboardPanel = new SuccessDashboardPanel({
                        model: {
                            dashboardToSave: this.model.dashboardToSave,
                            inmem: this.model.dasboardInmem,
                            application: this.model.application
                        }
                    });

                    this.model.inmem.on('createSuccess', function(){
                        this.children.create.$el.hide();
                        this.children.success.render().appendTo(this.$el);
                    }, this);

                    this.model.dasboardInmem.on('createSuccess', function(){
                        this.children.createDashboardPanel.$el.hide();
                        this.children.successDashboardPanel.$el.show();
                    }, this);

                    this.children.success.on('addToDashboardPanel', function(){
                        this.children.create.$el.hide();
                        this.children.success.$el.hide();
                        this.children.createDashboardPanel.render().appendTo(this.$el);
                        this.children.successDashboardPanel.$el.hide();
                    }, this);

                    this.on("hidden", function() {
                        if (!this.model.inmem.isNew()) {
                            this.model.report.fetch({url: splunkd_utils.fullpath(this.model.inmem.id)});
                        }
                    }, this);
                },
                render: function() {
                    this.children.create.render().appendTo(this.$el);
                    this.children.successDashboardPanel.render().appendTo(this.$el);
                    this.children.successDashboardPanel.$el.hide();

                    return this;
                }
        });
    }
);