define(
    [
        'module',
        'underscore',
        'views/Base',
        'util/splunkd_utils'
    ],
    function(
        module,
        _,
        BaseView,
        splunkdUtils
    )
    {
        return BaseView.extend({
            moduleId: module.id,
            tagName: 'form',
            className: 'search-query-control',
            /**
             * @param {Object} options {
             *     model: <models.State>
             * }
             */
            initialize: function(options) {
                BaseView.prototype.initialize.apply(this, arguments);
                this.key = this.options.key || "name";
                this.fetchDataFilter = this.options.fetchDataFilter || false;

                this.nativePlaceholder = this.supportsNativePlaceholder();
            },
            events: {
                'keyup input': function(e) {
                    e.preventDefault();
                    if (e.keyCode == 27) {
                        this.clear();
                    } else {
                        this.set(this.$input.val());
                    }
                    this.updateClear();
                    this.updatePlaceholder();
                },
                'click .placeholder': function(e) {
                    this.$input.focus();
                },
                'mouseup input': function(e) { //could result in pasted text
                    this.updatePlaceholder();
                },
                'submit': function(e) {
                    return false;
                },
                'click a.search-query-clear': function(e) {
                    e.preventDefault();
                    this.clear();
                    this.updateClear();
                    this.updatePlaceholder();
                }
            },
            clear: function(){
                this.$input.val('');
                this.set('');
            },
            updateClear: function(){
                this.$clear[this.$input.val() ? 'show' : 'hide']();
            },
            set: _.debounce(function(value) {
                //filter exists 
                if (value.length) {
                    this.model.state.set({
                        'offset': '0'
                    });

                    var keys = _.isArray(this.key) ? this.key : this.key.split(' ');
                    if(this.fetchDataFilter) {
                        //in the future, consumers of the shared input should refactor away
                        //from passing fetch data as the state model 
                        this.model.state.set('filter', splunkdUtils.createSearchFilterObj(value, keys));
                    } else {
                        var search = this.model.state.get('search'),
                            searchFilter = splunkdUtils.createSearchFilterString(value, keys);
                        if (search) {
                            _.chain(keys).map(function(key) {
                                return new RegExp('' + key + '="*.*\\*"');
                            },this)
                            .each(function(regex) {
                                if(regex.test(search))
                                    search = search.replace(regex, '');
                            },this);
                        } 
                        this.model.state.set('search', (search) ?
                            ([search, ' AND ', searchFilter].join('')): searchFilter);
                    }

                } else if(this.fetchDataFilter) {
                    this.model.state.unset('filter');
                } else if (search) {
                     this.model.state.unset('search');
                } else {
                    this.model.state.set('search', '');
                }
            }, 250),
            updatePlaceholder: function() {
                if (!this.nativePlaceholder) {
                    this.$placeholder[this.$input.val() === '' ? 'show' : 'hide']();
                }
            },
            supportsNativePlaceholder: function() {
                var test = document.createElement('input');
                return ('placeholder' in test);
            },
            isEmpty: function() {
                return this.$input.val() == '';
            },
            render: function() {
                this.$el.html(this.compiledTemplate({
                    _: _,
                    nativePlaceholder: this.nativePlaceholder
                }));
                if (!this.nativePlaceholder) {
                    this.$placeholder = this.$('.placeholder');
                }
                this.$clear = this.$('.search-query-clear');
                this.$input = this.$('input');
                return this;
            },
            template: '<input type="text" class="input-medium search-query" <% if(nativePlaceholder){ %>placeholder="<%-_("filter").t() %>"<%}%> >\
                <a href="#" class="search-query-clear" style="display:none"><i class="icon-x-circle"></i></a>\
                <% if(!nativePlaceholder){ %> <span class="placeholder"><%= _("filter").t() %></span><% } %>'
        });
    }
);
