define(
    [
        'underscore',
        'module',
        'models/classicurl',
        'views/Base',
        'views/virtual_indexes/IndexesTab',
        'views/virtual_indexes/ProvidersTab',
        'views/virtual_indexes/UserImpersonationTab',
        'views/virtual_indexes/DeleteDialog',
        'contrib/text!views/virtual_indexes/Master.html',
        'uri/route',
        'bootstrap.tab'
    ],
    function(
        _,
        module,
        classicurlModel,
        BaseView,
        IndexesTabView,
        ProvidersTabView,
        UserImpersonationTabView,
        DeleteDialog,
        MasterTemplate,
        route,
        undefined
        ){
        return BaseView.extend({
            moduleId: module.id,
            template: MasterTemplate,
            /**
             * @param {Object} options {
             *
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);

                this.children.indexesTabView = new IndexesTabView({
                    collection: this.collection,
                    model: this.model
                });
                this.children.providersTabView = new ProvidersTabView({
                    collection: this.collection,
                    model: this.model
                });
                this.children.userImpersonationTabView = new UserImpersonationTabView({
                    collection: this.collection,
                    model: this.model,
                    providersDfd: this.options.providersDfd,
                    userImpersonationDfd: this.options.userImpersonationDfd,
                    splunkUsersDfd: this.options.splunkUsersDfd
                });

                this.children.deleteDialog = new DeleteDialog({
                    model: this.model
                });

                this.collection.providersAll.on('change reset', function(){
                    this.updateProvidersCount(this.$el);
                }, this);
                this.collection.indexesAll.on('change reset', function(){
                    this.updateIndexesCount(this.$el);
                }, this);

                this.collection.providers.on('deleteRequest', function(victimModel) {
                    this.children.deleteDialog = new DeleteDialog({
                        collection: this.collection.providers,
                        model: victimModel,
                        onHiddenRemove: true
                    });
                    $('body').append(this.children.deleteDialog.render().el);
                    this.children.deleteDialog.show();
                }, this);
                this.collection.indexes.on('deleteRequest', function(victimModel) {
                    this.children.deleteDialog = new DeleteDialog({
                        collection: this.collection.indexes,
                        model: victimModel,
                        onHiddenRemove: true
                    });
                    $('body').append(this.children.deleteDialog.render().el);
                    this.children.deleteDialog.show();
                }, this);

                this.collection.indexes.on('search', function(search) {
                    var url = route.search(
                        this.model.application.get('root'),
                        this.model.application.get('locale'),
                        'search',
                        {
                            data: {
                                q: 'index='+search
                            }
                        }
                    );
                    window.open(url,'_blank');
                }, this);
            },
            events: {
                'click #providers-tab a': function() {
                    classicurlModel.save({t: 'providers'});
                },
                'click #indexes-tab a': function() {
                    classicurlModel.save({t: 'indexes'});
                },
                'click #impersonation-tab a': function() {
                    classicurlModel.save({t: 'impersonation'});
                }
            },
            updateProvidersCount: function($html){
                var el = $html.find('.provider_count');
                el.text(this.collection.providersAll.length);
            },
            updateIndexesCount: function($html){
                var el = $html.find('.indexes_count');
                el.text(this.collection.indexesAll.length);
            },
            makeDocLink: function(location) {
                return route.docHelp(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    location
                );
            },
            render: function() {
                var urlDfd = $.Deferred();
                classicurlModel.fetch({
                    success: function(model, response) {
                        var t = classicurlModel.get('t');
                        urlDfd.resolve({t:t});
                    }
                });
                var html = this.compiledTemplate({
                    provider_count: this.collection.providersAll.length,
                    indexes_count: this.collection.indexesAll.length,
                    docLink: this.makeDocLink('manager.virtualindex.dashboard')
                    }),
                $html = $(html);

                // open tab requested by the url
                $.when(urlDfd).done(function(args) {
                    if ($html.filter('.nav-tabs').find('a[href="#vix_'+args.t+'"]').length == 0) {
                        $html.find('#vix_providers').addClass('active');
                        return;
                    }
                    $html.filter('.nav-tabs').find('a[href="#vix_'+args.t+'"]').tab('show');
                    $html.find('#vix_'+args.t).addClass('active');
                });

                $html.find('#vix_indexes').append(this.children.indexesTabView.render().el);
                $html.find('#vix_providers').append(this.children.providersTabView.render().el);
                $html.find('#vix_impersonation').append(this.children.userImpersonationTabView.render().el);
                this.$el.html($html);

                return this;
            }

        });
    }
);
