define([
    'underscore',
    'module',
    'views/Base',
    'views/virtual_indexes/ProvidersGrid',
    'views/shared/CollectionPaginator',
    'views/shared/tablecaption/Input',
    'views/shared/controls/SyntheticSelectControl',
    'views/shared/FlashMessages'
],
    function(
        _,
        module,
        BaseView,
        ProvidersGrid,
        Paginator,
        Filter,
        SyntheticSelectControl,
        FlashMessagesView
        ){
        return BaseView.extend({
            moduleId: module.id,
            className: 'section-padded',
            initialize: function(options){
                BaseView.prototype.initialize.call(this, options);

                this.children.filter = new Filter({
                    className: 'pull-left',
                    model: {
                        state: this.collection.providers.fetchData
                    }
                });
                // Select the number of results per page
                this.children.pageCount = new SyntheticSelectControl({
                    menuWidth: "narrow",
                    className: "btn-group pull-left",
                    items: [
                        {value: 10, label: _('10 per page').t()},
                        {value: 20, label: _('20 per page').t()},
                        {value: 50, label: _('50 per page').t()},
                        {value: 100, label: _('100 per page').t()}
                    ],
                    model: this.collection.providers.fetchData,
                    modelAttribute: 'count',
                    toggleClassName: 'btn-pill'
                });

                this.children.paginator = new Paginator({
                    collection: this.collection.providers
                });
                this.children.providersGrid = new ProvidersGrid({
                    collection: this.collection,
                    model: this.model
                });

                // Prepare a flashmessage box for a possible delete error message
                this.collection.providers.on('deleteRequest', function(victimModel) {
                    if (this.children.flashMessages) {
                        this.children.flashMessages.remove();
                    }
                    this.children.flashMessages = new FlashMessagesView({ model: victimModel });
                    this.$el.prepend(this.children.flashMessages.render().el);
                }, this);

                this.collection.providers.on('change reset', function(){
                    if (this.collection.providers.length == 0) {
                        this.children.filter.isEmpty() ? this.children.filter.$el.hide() : '';
                        this.children.paginator.$el.hide();
                        this.children.pageCount.$el.hide();
                        this.$('.divider').hide();
                    } else {
                        this.children.filter.$el.show();
                        this.children.paginator.$el.show();
                        this.children.pageCount.$el.show();
                        this.$('.divider').show();
                    }
                }, this);

            },
            render: function() {
                // ERP-285: New virtual index/New provider buttons doubled/tripled if use browser's Back/Forward buttons
                this.$el.empty();
                this.$el.append(this.children.filter.render().el);
                this.$el.append('<a href="vix_provider_new" class="btn btn-primary pull-right">'+_('New provider').t()+'</a>');
                this.$el.append('<div class="divider" />');
                this.$el.append(this.children.pageCount.render().el);
                this.$el.append(this.children.paginator.render().el);
                this.$el.append(this.children.providersGrid.el);


                return this;
            }
        });
    });
