define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'collections/shared/TimeZones',
        'views/shared/controls/TimeZone',
        'views/virtual_indexes/custom_controls/TimeSecondsControl',
        'views/shared/controls/ControlGroup',
        'views/shared/controls/Section',
        'views/shared/FlashMessages',
        'views/shared/DynamicFields',
        'contrib/text!views/virtual_indexes/config/IndexSetup.html',
        'uri/route',
        'util/general_utils'
    ],
    function(
        $,
        _,
        Backbone,
        module,
        BaseView,
        TimeZoneTable,
        TimeZoneControl,
        TimeSecondsControl,
        ControlGroup,
        Section,
        FlashMessagesView,
        DynamicFieldsView,
        Template,
        route,
        utils
        ){
        return BaseView.extend({
            moduleId: module.id,
            template: Template,
            /**
             * @param {Object} options {
             *
             * }
             */
            initialize: function() {
                BaseView.prototype.initialize.apply(this, arguments);
                this.formModel = new Backbone.Model();

                this.sections = {};
                this.providersDfd = $.Deferred();
                this.isEdit = false;

                this.children.flashMessages = new FlashMessagesView({ model: this.model.index });
                var that = this;

                // Default settings
                this.formModel.set({recursiveProcess: true});

                this.children.name = new ControlGroup({
                    className: 'index-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'name',
                        model: this.model.index.entry,
                        save: false
                    },
                    required: true,
                    label: _('Name').t()
                });

                this.collection.providers.on('reset change', function() {
                    this.providersDfd.resolve();
                    this.providerNames = this.collection.providers.map(function(model) {
                        return {label:model.entry.get('name'), value:model.entry.get('name')};
                    });
                    this.children.provider = new ControlGroup({
                        className: 'index-provider control-group',
                        controlType: 'SyntheticSelect',
                        controlClass: 'controls-block',
                        controlOptions: {
                            modelAttribute: 'vix.provider',
                            model: this.model.index.entry.content,
                            items: this.providerNames,
                            className: 'btn-group view-count',
                            menuWidth: 'narrow',
                            toggleClassName: 'btn'
                        },
                        label: _("Provider").t()
                    });
                    // notify the section of loading complete
                    this.formModel.trigger('update', 'provider');

                    // make the first element submitted by default
                    if (!this.model.index.entry.content.get('vix.provider') && this.providerNames && this.providerNames.length>0) {
                        this.model.index.entry.content.set('vix.provider', this.providerNames[0].value);
                    }
                    if (this.providerNames.length <= 1) {
                        this.children.provider.disable();
                    } else {
                        this.children.provider.enable();
                    }

                }, this);

                this.children.pathHdfs = new ControlGroup({
                    className: 'index-path-hdfs control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.input.1.path',
                        model: this.model.index.entry.content,
                        save: false
                    },
                    label:   _('Path to data in HDFS').t(),
                    help:    _('Example: /home/data/apache/logs/').t(),
                    tooltip: _('Root path to the data that belongs in this virtual index').t()
                });

                this.children.recursiveProcess = new ControlGroup({
                    className: 'index-recursive control-group',
                    controlType: 'SyntheticCheckbox',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'recursiveProcess',
                        model: this.formModel,
                        save: false
                    },
                    label: _('Recursively process the directory').t()
                });

                this.children.whitelist = new ControlGroup({
                    className: 'index-whitelist control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.input.1.accept',
                        model: this.model.index.entry.content,
                        save: false
                    },
                    label:   _('Whitelist').t(),
                    help:    _('Regex that matches the file path. Example: \\.gz$').t(),
                    tooltip: _('Regular expression that file paths that belong in this virtual index should match').t()
                });

                this.children.customizeTimestamp = new ControlGroup({
                    className: 'index-whitelist control-group',
                    controlType: 'SyntheticCheckbox',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'customizeTimestamp',
                        model: this.formModel,
                        save: false
                    },
                    label: _('Customize timestamp format').t()
                });

                this.children.timeRegex = new ControlGroup({
                    className: 'index-time-regex control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.input.1.et.regex',
                        model: this.model.index.entry.content,
                        save: false
                    },
                    label:   _('Time capturing regex').t(),
                    help:    _('Regex to capture date/time from path. Example: /home/data/(\\d+)/(\\d+)/').t(),
                    tooltip: _('Regex with capturing groups to extract time components from paths. The capture groups are concatenated and interpreted using the format below').t() 
                });

                this.children.timeFormat = new ControlGroup({
                    className: 'index-time-format control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.input.1.et.format',
                        model: this.model.index.entry.content,
                        save: false
                    },
                    label:   _('Time Format').t(),
                    help:    _('Format to interpret the extracted time string as. Example: yyyyMMddHH').t(),
                    tooltip: _('Format string to interpret the time string extracted from the path. The format is Java\'s SimpleDateFormat').t()
                });

                this.children.timeOffset = new ControlGroup({
                    className: 'control-group',
                    controlClass: 'controls-block',
                    controlType: 'TimeSecondsControl',
                    controlTypes: {'TimeSecondsControl': TimeSecondsControl},
                    controlOptions: {
                        modelAttribute: 'vix.input.1.et.offset',
                        model: this.model.index.entry.content,
                        save: false
                    },
                    label: _('Time Adjustment').t(),
                    help: _('Earliest time offset (1h, 1d, etc.)').t()
                });

                this.children.timeRange = new ControlGroup({
                    className: 'control-group',
                    controlClass: 'controls-block',
                    controlType: 'TimeSecondsControl',
                    controlTypes: {'TimeSecondsControl': TimeSecondsControl},
                    controlOptions: {
                        modelAttribute: 'vix.input.1.lt.offset',
                        model: this.model.index.entry.content,
                        save: false
                    },
                    label: _('Time Range').t(),
                    help: _('How much data is contained in the dir (1h, 1d, etc.)').t()
                });

                this.children.timeZone = new ControlGroup({
                    className: 'index-time-offset control-group',
                    controlClass: 'controls-block',
                    controlType: 'TimeZone',
                    controlTypes: {'TimeZone': TimeZoneControl},
                    controlOptions: {
                        modelAttribute: 'vix.input.1.et.timezone',
                        model: this.model.index.entry.content,
                        save: false
                    },
                    label: _('Time Zone').t()
                });

                /**
                 * settings based on timezone change. 
                 * ERP-392 Virtual Index UI: Simplify time/latest time extraction rules
                 */

                this.listenTo(
                    this.model.index.entry.content, 
                    'change:vix.input.1.et.regex reset:vix.input.1.et.regex ' +
                    'change:vix.input.1.et.format reset:vix.input.1.et.format ' +
                    'change:vix.input.1.et.timezone reset:vix.input.1.et.timezone',
                    function () {
                        var regex = this.model.index.entry.content.get('vix.input.1.et.regex');
                        var format = this.model.index.entry.content.get('vix.input.1.et.format');
                        var timezone = this.model.index.entry.content.get('vix.input.1.et.timezone');

                        this.model.index.entry.content.set({
                            'vix.input.1.lt.regex': regex,
                            'vix.input.1.lt.format': format,
                            'vix.input.1.lt.timezone': timezone
                        });
                });

                this.formModel.on('loadComplete', function() {
                    this.children.dynamicFields = new DynamicFieldsView({
                        model: this.model.index.entry.content,
                        prefix: 'vix.',
                        exclude: this._getChildrenAttributes(),
                        hideEmpty: true,
                        silent: true
                    });
                    this.formModel.trigger('update', 'dynamicFields');
                }, this);

                /*
                 * Sections
                 */

                this.sections.basic = new Section({
                    label: '',
                    id: 'basic',
                    children: this.children,
                    model: {
                        form: this.formModel,
                        data: this.model.provider
                    },
                    order: ['name', 'provider']
                });
                this.sections.paths = new Section({
                    label: _('Paths').t(),
                    id: 'paths',
                    children: this.children,
                    model: {
                        data: this.model.index
                    },
                    order: ['pathHdfs', 'recursiveProcess', 'whitelist']
                });
                this.sections.time = new Section({
                    label: _('Time').t(),
                    id: 'time',
                    children: this.children,
                    model: {
                        data: this.model.index
                    },
                    order: ['timeRegex', 'timeFormat', 'timeOffset', 'timeRange', 'timeZone']
                });

                this.sections.additionalSettings = new Section({
                    label: _('Settings').t(),
                    id: 'additional-settings',
                    description: _('').t(),
                    children: this.children,
                    model: {
                        data: this.model.index
                    },
                    order: ['dynamicFields']
                });
                // avoiding circular dependency
                this.sections.additionalSettings.loadStatusDfd.resolve();

                $.extend(this.children, this.sections);

                // trigger 'loadComplete' when all the sections reportedly finish loading
                var sectionDfds = _.map(this.sections, function(section) {
                    return section.getLoadStatusDfd();
                }, this);
                $.when.apply($, sectionDfds).then(function() {
                    this.formModel.trigger('loadComplete');
                }.bind(this));

                // if any of the customizeTimestamp group members is not empty, expand the group
                this.model.index.on('change reset', function() {
                    if (this.isEdit) {
                        // set the process recursively checkbox if ellipsis is found in the input path
                        var hdfsPath = this.model.index.entry.content.get('vix.input.1.path');
                        if (utils.strEndsWith(hdfsPath, '/...')) {
                            hdfsPath = hdfsPath.slice(0, hdfsPath.lastIndexOf('/'));
                            this.model.index.entry.content.set({'vix.input.1.path': hdfsPath});
                            this.formModel.set({recursiveProcess: true});
                        } else {
                            this.formModel.set({recursiveProcess: false});
                        }
                    } else {
                        this.model.index.entry.set('name','');
                    }
                    // preserve events in subviews while rerendering
                    var adminContent = this.$('.adminContent').detach();
                    this.render();
                    this.$('.adminContent').replaceWith(adminContent);

                    var groupHasValues = !!_.find(['time'], function(section) {
                        if (!this.sections.hasOwnProperty(section)) {
                            return false;
                        }
                        return !!_.find(this.sections[section].getChildren(), function(child) {
                            if (typeof child.getModelAttributes == 'function') {
                                var attr = child.getModelAttributes()[0],
                                    val = this.model.index.entry.content.get(attr);
                                return val;
                            }
                            return false;
                        }, this);
                    }, this);
                    this.formModel.set('customizeTimestamp', groupHasValues);
                }, this);

                // bind formModel events to DOM changes
                this.formModel.on('change:customizeTimestamp', function() {
                    if (this.formModel.get('customizeTimestamp')) {
                        this.sections.time.show(true);
                    } else {
                        this.sections.time.hide(true);
                    }
                }, this);
                this.formModel.set('customizeTimestamp', 0);

                // the dropdown associated with this model attribute toggles visibility of several sections
                this.model.index.entry.content.on('change:vix.provider', function() {
                    var providerName = that.model.index.entry.content.get('vix.provider');
                    this.providersDfd.done(function() {
                        var provider = that.collection.providers.find(function(prov) {
                            return prov.entry.get('name') == providerName;
                        }),
                        family = provider.entry.content.get('vix.family');
                        if (family == 'hadoop') {
                            that.children.paths.show(true);
                            that.children.customizeTimestamp.show();
                            if (that.formModel.get('customizeTimestamp')) {
                                that.sections.time.show(true);
                            } else {
                                that.sections.time.hide(true);
                            }
                        } else {
                            that.children.paths.hide(true);
                            that.children.customizeTimestamp.hide();
                            that.sections.time.hide(true);
                        }

                    });

                }, this);


                this.model.application.on('change reset', function() {
                    this.returnUrl = route.manager(
                        this.model.application.get('root'),
                        this.model.application.get('locale'),
                        'system',
                        'virtual_indexes',
                        {
                            data:
                            {
                                t: 'indexes'
                            }
                        }
                    );
                }, this);
            },
            events: {
                'click .btn-primary': function(e) {
                    e.preventDefault();
                    // name is in entry when we're getting its value and it should be in entry.content for saving it
                    this.model.index.entry.content.set({'name': this.model.index.entry.get('name')}, {silent: true});

                    // smart append ellipsis to the path when process recursively checkbox is checked
                    var inputPath = this.model.index.entry.content.get('vix.input.1.path');
                    if (inputPath && this.formModel.has('recursiveProcess')) {
                        if (this.formModel.get('recursiveProcess')) {
                            if (utils.strEndsWith(inputPath, '/')) {
                                inputPath += '...';
                            } else if (!utils.strEndsWith(inputPath, '...')) {
                                inputPath += '/...';
                            }
                            this.model.index.entry.content.set({'vix.input.1.path': inputPath}, {silent: true});
                        }
                    }

                    if (!this.formModel.get('customizeTimestamp')) {
                        // if extended form is hidden, clear its values
                        this.sections.time.clear();
                    }

                    this.model.index.save({}, {silent: true}).done(function(){
                        window.location.href = this.returnUrl;
                    }.bind(this));
					$('html, body').animate({ scrollTop: 0 }, 'fast');
                },
                'click .cancel': function(e) {
                    e.preventDefault();
                    window.location.href = this.returnUrl;
                }
            },

            /*
             For 'Additional settings' we want to get all the model attributes that are not included into any of the ControlGroups.
             This method returns the list of attributes already used that should be excluded.
             */
            _getChildrenAttributes: function() {
                var results = _.map(this.children, function(child) {
                    if (child.hasOwnProperty('childList')) {
                        return child.childList[0].getModelAttribute();
                    }
                    return '';
                });
                // Need to hardcode the latest time settings since we copied them from the earliest time values
                results.push('vix.input.1.lt.regex', 'vix.input.1.lt.format', 'vix.input.1.lt.timezone');
                return results;
            },
            managerRoute: function(page, options) {
                return route.manager(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'system',
                    page,
                    options
                );
            },
            render: function() {
                this.isEdit = (this.model.index.get('mode') == 'edit');
                var html = this.compiledTemplate({
                    isEdit: this.isEdit,
                    managerRoute: this.managerRoute.bind(this)
                });

                var $html = $(html);
                if (!this.el.innerHTML) {
                    $html.find('.adminContent').append(this.children.flashMessages.render().el);
                    $html.find('.adminContent').append(this.children.basic.render().el);
                    $html.find('.adminContent').append(this.children.paths.render().el);
                    $html.find('.adminContent').append(this.children.customizeTimestamp.render().el);
                    $html.find('.adminContent').append(this.children.time.render().el);
                    $html.find('.adminContent').append(this.children.additionalSettings.render().el);

                    $html.find('.adminContent').append('<div class="button-wrapper" />');
                    $html.find('.adminContent .button-wrapper').append('<a href="#" class="btn cancel pull-left">' + _('Cancel').t() + '</a>');
                    $html.find('.adminContent .button-wrapper').append('<a href="#" class="btn btn-primary pull-right">' + _('Save').t() + '</a>');
                }
                this.$el.html($html);

                if (this.isEdit) {
                    this.children.name.childList[0].disable();
                }

                return this;
            }

        });
    }
);
