define(
    [
        'jquery',
        'underscore',
        'backbone',
        'module',
        'views/Base',
        'views/shared/controls/ControlGroup',
        'views/shared/controls/Section',
        'views/shared/FlashMessages',
        'views/shared/DynamicFields',
        'contrib/text!views/virtual_indexes/config/ProviderSetup.html',
        'uri/route'
    ],
    function(
        $,
        _,
        Backbone,
        module,
        BaseView,
        ControlGroup,
        Section,
        FlashMessagesView,
        DynamicFieldsView,
        Template,
        route
        ){
        var HADOOP_1X = '$SPLUNK_HOME/bin/jars/SplunkMR-s6.0-h1.0.jar';
        var HADOOP_2X = '$SPLUNK_HOME/bin/jars/SplunkMR-s6.0-h2.0.jar';
        var HADOOP_2X_YARN = '$SPLUNK_HOME/bin/jars/SplunkMR-s6.0-hy2.0.jar';
        var HADOOP_VERSION = 'vix.command.arg.3';
        var JOB_TRACKER = 'vix.mapred.job.tracker';
        var KERBEROS_AUTHORIZATION = 'vix.hadoop.security.authorization';
        var KERBEROS_MODE = 'vix.hadoop.security.authentication';
        var KERBEROS_MODE_DEFAULT = 'kerberos';
        var KERBEROS_SERVER_NAME = 'vix.javaprops.java.security.krb5.kdc';
        var KERBEROS_REALM = 'vix.javaprops.java.security.krb5.realm';
        var KERBEROS_MAPREDUCE_PRINCIPAL = 'vix.mapreduce.jobtracker.kerberos.principal';
        var OLD_KERBEROS_SERVER_NAME = 'vix.java.security.krb5.kdc';
        var OLD_KERBEROS_REALM = 'vix.java.security.krb5.realm';
        var KERBEROS_CONFIGURATION_FILE_PATH = 'vix.javaprops.java.security.krb5.conf';
        var KERBEROS_CONFIG_SWITCH = 'kerberos.config';
        var KERBEROS_CONFIG_SWITCH_SERVER_NAME_AND_REALM = '1';
        var KERBEROS_CONFIG_SWITCH_FILE_PATH = '2';

        return BaseView.extend({
            moduleId: module.id,
            template: Template,
            /**
             * @param {Object} options {
             *
             * }
             */
            initialize: function() {

                // TODO: change Kerberos section into two sections ERP-582
                BaseView.prototype.initialize.apply(this, arguments);

                this.formModel = new Backbone.Model(); // event bus
                this.sections = {};
                this.isEdit = false;
                this.formLoadDfd = $.Deferred();

                this._initFlashMessage();
                this._initBasic();
                this._initEnvVars();
                this._initProviderPaths();
                this._initSplunkSettings();
                this._initKerberosSettings();
                this._initAdditionalSettings();

                // this is for consistency since sections are child views too
                $.extend(this.children, this.sections);

                // trigger 'loadComplete' when all the sections reportedly finish loading
                var sectionDfds = _.map(this.sections, function(section) {
                    return section.getLoadStatusDfd();
                }, this);
                $.when.apply($, sectionDfds).then(function(schemas) {
                    this.formModel.trigger('loadComplete');
                }.bind(this));

                // the dropdown associated with this model attribute toggles visibility of several sections
                this.model.provider.entry.content.on('change:vix.family', this._onChangeFamily, this);

                this.model.application.on('change', function() {
                    this.returnUrl = route.manager(
                        this.model.application.get('root'),
                        this.model.application.get('locale'),
                        'system',
                        'virtual_indexes',
                        {
                            data:
                            {
                                t: 'providers'
                            }
                        }
                    );
                }, this);

                this.model.provider.on('change', function() {
                    if (!this.isEdit) {
                        this.model.provider.entry.set('name','');
                    }
                    // preserve events in subviews while rerendering
                    var adminContent = this.$('.adminContent').detach();
                    this.render();
                    this.$('.adminContent').replaceWith(adminContent);
                }, this);

                this.listenTo(this.model.provider.entry.content,
                    'change:' + KERBEROS_AUTHORIZATION,
                    this._toggleKerberosSectionAndMode);

                /**
                 * ERP-391: when user chooses 'Hadoop 2.x, (Yarn)', we remove 'Job Tracker' field and add three more
                 * attributes in the additional settings.
                 */
                this.listenTo(this.model.provider.entry.content,
                    'change:' + HADOOP_VERSION,
                    this._onChangeHadoopVersion);

                this.listenTo(this.model.provider.entry.content,
                    'change:' + OLD_KERBEROS_SERVER_NAME + ' change:' + OLD_KERBEROS_REALM,
                    this._syncOldKerberosAttributes);

                this.listenTo(this.formModel,
                    'change:' + KERBEROS_CONFIG_SWITCH,
                    this._onChooseKerberosConfigSwitch);
            },

            events: {
                'click .btn-primary': function(e) {
                    e.preventDefault();
                    this.model.provider.entry.content.set('name', this.model.provider.entry.get('name'));

                    this.model.provider.save({}, {silent: true}).done(function(){
                        window.location.href = this.returnUrl;
                    }.bind(this));
                    $('html, body').animate({ scrollTop: 0 }, 'fast');
                },
                'click .cancel': function(e) {
                    e.preventDefault();
                    window.location.href = this.returnUrl;
                }
            },

            /*
             For 'Additional settings' we want to get all the model attributes that are not included into any of the ControlGroups.
             This method returns the list of attributes already used that should be excluded.
             */
            _getChildrenAttributes: function() {
                return _.map(this.children, function(child) {
                    if (child.hasOwnProperty('childList')) {
                        return child.childList[0].getModelAttribute();
                    }
                    return '';
                });
            },

            _managerRoute: function(page, options) {
                return route.manager(
                    this.model.application.get('root'),
                    this.model.application.get('locale'),
                    'system',
                    page,
                    options
                );
            },

            _initFlashMessage: function() {
                this.children.flashMessages = new FlashMessagesView({ model: this.model.provider });
            },

            _initBasic: function() { // Name and family
                this.children.name = new ControlGroup({
                    className: 'prov-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'name',
                        model: this.model.provider.entry,
                        save: false
                    },
                    required: true,
                    label: _('Name').t()
                });

                this.collection.indexesConf.on('change reset', function() {
                    this.familyNames = this.collection.indexesConf.map(function(model) {
                        var name = model.entry.get('name').match(/provider-family:(.*)/)[1];
                        return {label: name, value: name};
                    });
                    this.children.family = new ControlGroup({
                        className: 'prov-family control-group',
                        controlType: 'SyntheticSelect',
                        controlClass: 'controls-block',
                        controlOptions: {
                            modelAttribute: 'vix.family',
                            model: this.model.provider.entry.content,
                            items: this.familyNames,
                            className: 'btn-group view-count',
                            menuWidth: 'narrow',
                            toggleClassName: 'btn'
                        },
                        label: _("Provider Family").t()
                    });
                    // notify the section of loading complete
                    this.formModel.trigger('update', 'family');

                    // make the first element submitted by default
                    if (!this.model.provider.entry.content.get('vix.family') && this.familyNames && this.familyNames.length>0) {
                        this.model.provider.entry.content.set('vix.family', this.familyNames[0].value);
                    }

                    if (this.familyNames.length <= 1) {
                        this.children.family.disable();
                    } else {
                        this.children.family.enable();
                    }
                }, this);

                this.sections.basic = new Section({
                    label: '',
                    id: 'basic',
                    children: this.children,
                    model: {
                        form: this.formModel,
                        data: this.model.provider
                    },
                    order: ['name', 'family']
                });
            },

            _initEnvVars: function() {
                this.children.javaHome = new ControlGroup({
                    className: 'prov-java-home control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.env.JAVA_HOME',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Java Home').t(),
                    help: _('Example: /usr/jdk').t()
                });

                this.children.hadoopHome = new ControlGroup({
                    className: 'prov-hadoop-home control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.env.HADOOP_HOME',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Hadoop Home').t(),
                    help: _('Example: /usr/hadoop').t()
                });

                this.sections.envVars = new Section({
                    label: _('Environment Variables').t(),
                    id: 'env-vars',
                    children: this.children,
                    model: {
                        data: this.model.provider
                    },
                    order: ['javaHome', 'hadoopHome']
                });
            },

            _initProviderPaths: function() {
                this.children.hadoopVersion = new ControlGroup({
                    className: 'prov-hadoop-version control-group',
                    controlType: 'SyntheticSelect',
                    controlOptions: {
                        modelAttribute: HADOOP_VERSION,
                        model: this.model.provider.entry.content,
                        items:[
                            {label: _('Hadoop 1.x, (MRv1)').t(), value: HADOOP_1X},
                            {label: _('Hadoop 2.x, (MRv1)').t(), value: HADOOP_2X},
                            {label: _('Hadoop 2.x, (Yarn)').t(), value: HADOOP_2X_YARN}
                        ],
                        className: 'btn-group view-count',
                        menuWidth: 'narrow',
                        toggleClassName: 'btn'
                    },
                    label: _("Hadoop Version").t()
                });

                this.children.jobTracker = new ControlGroup({
                    className: 'prov-job-tracker control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: JOB_TRACKER,
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Job Tracker').t(),
                    help: _('Example: jobtracker.example.com:8021').t()
                });

                this.children.fileSystem = new ControlGroup({
                    className: 'prov-file-system control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.fs.default.name',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('File System').t(),
                    help: _('Example: hdfs://namenode.example.com:8020').t()
                });

                this.children.resourceManager = new ControlGroup({
                    className: 'prov-resource-manager control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.yarn.resourcemanager.address',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Resource Manager Address').t(),
                    help: _('').t()
                });

                this.children.resourceScheduler = new ControlGroup({
                    className: 'prov-resource-scheduler control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.yarn.resourcemanager.scheduler.address',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Resource Scheduler Address').t(),
                    help: _('').t()
                });

                this.sections.providerPaths = new Section({
                    label: _('Hadoop Cluster Information').t(),
                    id: 'provider-paths',
                    children: this.children,
                    model: {
                        data: this.model.provider
                    },
                    order: ['hadoopVersion', 'jobTracker', 'fileSystem', 'resourceManager', 'resourceScheduler']
                });
            },

            _initSplunkSettings: function() {
                this.children.homeHdfs = new ControlGroup({
                    className: 'prov-home-hdfs control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.splunk.home.hdfs',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('HDFS Working Directory').t(),
                    help: _('Example: /user/splunk/hunk-01.example.com/').t(),
                    tooltip: _('Path in HDFS that you want this provider to use as it\'s workspace').t()
                });

                this.children.jobQueue = new ControlGroup({
                    className: 'prov-job-queue control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.mapred.job.queue.name',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Job Queue').t(),
                    help: _('Example: highPriorityQ').t(),
                    tooltip: _('Job queue where MapReduce jobs of this provider get submitted to').t()
                });

                this.sections.splunkSettings = new Section({
                    label: _('Splunk Settings').t(),
                    id: 'splunk-settings',
                    children: this.children,
                    model: {
                        data: this.model.provider
                    },
                    order: ['homeHdfs', 'jobQueue']
                });
            },

            _initKerberosSettings: function() {
                // this is to control Kerberos Settings show/hide
                this.children.addSecureCluster = new ControlGroup({
                    className: 'prov-secure-cluster control-group',
                    controlType: 'SyntheticCheckbox',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: KERBEROS_AUTHORIZATION,
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Add Secure Cluster').t()
                });

                this.children.kerberosMode = new ControlGroup({
                    className: 'prov-kerberos-mode control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: KERBEROS_MODE,
                        model: this.model.provider.entry.content,
                        placeholder: KERBEROS_MODE_DEFAULT,
                        useSyntheticPlaceholder: true,
                        defaultValue: KERBEROS_MODE_DEFAULT,
                        save: false
                    },
                    label: _('Mode').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                this.children.kerberosConfig = new ControlGroup({
                    className: 'prov-kerberos-config control-group',
                    controlType: 'SyntheticSelect',
                    controlOptions: {
                        modelAttribute: KERBEROS_CONFIG_SWITCH,
                        model: this.formModel,
                        items:[
                            {label: _('------ Choose ------ ').t(), value: null, enabled: false},
                            {label: _('Server Name and Realm').t(), value: KERBEROS_CONFIG_SWITCH_SERVER_NAME_AND_REALM},
                            {label: _('Configuration File Path').t(), value: KERBEROS_CONFIG_SWITCH_FILE_PATH}
                        ],
                        className: 'btn-group view-count',
                        toggleClassName: 'btn'
                    },
                    label: _('').t()
                });

                this.children.kerberosServerName = new ControlGroup({
                    className: 'prov-kerberos-server-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: KERBEROS_SERVER_NAME,
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Kerberos Server Name').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                this.children.kerberosDefaultRealm = new ControlGroup({
                    className: 'prov-kerberos-default-realm control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: KERBEROS_REALM,
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Kerberos Default Realm').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                // ERP-582: Configuration File Path
                this.children.kerberosConfigurationFilePath = new ControlGroup({
                    className: 'prov-kerberos-configuration-file-path control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: KERBEROS_CONFIGURATION_FILE_PATH,
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Kerberos Configuration File Path').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                this.children.kerberosPrincipalName = new ControlGroup({
                    className: 'prov-kerberos-principal-name control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.kerberos.principal',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Kerberos Principal Name').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                this.children.kerberosKeytabPath = new ControlGroup({
                    className: 'prov-kerberos-keytab-path control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.kerberos.keytab',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Kerberos Keytab Path').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                this.children.hdfsPrincipal = new ControlGroup({
                    className: 'prov-hdfs-principal control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.dfs.namenode.kerberos.principal',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('HDFS Principal').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                this.children.mapreducePrincipal = new ControlGroup({
                    className: 'prov-mapreduce-principal control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: KERBEROS_MAPREDUCE_PRINCIPAL,
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('MapReduce Principal').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                this.children.resourcemanagerPrincipal = new ControlGroup({
                    className: 'prov-resourcemanager-principal control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.yarn.resourcemanager.principal',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Resource Manager Principal').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                this.children.nodemanagerPrincipal = new ControlGroup({
                    className: 'prov-nodemanager-principal control-group',
                    controlType: 'Text',
                    controlClass: 'controls-block',
                    controlOptions: {
                        modelAttribute: 'vix.yarn.nodemanager.principal',
                        model: this.model.provider.entry.content,
                        save: false
                    },
                    label: _('Node Manager Principal').t(),
                    help: _('').t(),
                    tooltip: _('').t()
                });

                // Section
                this.sections.kerberosSettings = new Section({
                    label: _('Security Settings').t(),
                    id: 'kerberos-settings',
                    children: this.children,
                    model: {
                        data: this.model.provider.entry.content
                    },
                    order: ['kerberosMode', 'kerberosConfig', 'kerberosServerName', 'kerberosDefaultRealm', 'kerberosConfigurationFilePath', 'kerberosPrincipalName',
                        'kerberosKeytabPath', 'hdfsPrincipal', 'mapreducePrincipal', 'resourcemanagerPrincipal',
                        'nodemanagerPrincipal']
                });

            },

            _initAdditionalSettings: function() {
                // wait for all the controls to finish loading before getting the list of model attributes bound for exclusion
                this.formModel.on('loadComplete', function() {
                    this.children.dynamicFields = new DynamicFieldsView({
                        model: this.model.provider.entry.content,
                        prefix: 'vix.',
                        exclude: this._getChildrenAttributes(),
                        hideEmpty: true,
                        silent: true
                    });
                    this.formModel.trigger('update', 'dynamicFields');
                    this.formLoadDfd.resolve();
                }, this);

                this.sections.additionalSettings = new Section({
                    label: _('Additional Settings').t(),
                    id: 'additional-settings',
                    description: _('Additional settings for hadoop configuration. Refer to Hadoop mapred documentation for setting name and description.').t(),
                    children: this.children,
                    order: ['dynamicFields']
                });

                // avoiding circular dependency
                this.sections.additionalSettings.loadStatusDfd.resolve();
            },

            _onChangeHadoopVersion: function() {
                // choose 'yarn'
                if (this.isYarn()) {
                    /** Hadoop Cluster Information section:
                     *   - remove job tracker
                     *   - add Resource Manager and Resource Scheduler
                     */
                    this.setJobTracker('');
                    this.children.jobTracker.$el.hide('fast');
                    this.children.resourceManager.$el.show('fast');
                    this.children.resourceScheduler.$el.show('fast');

                    /** Additional Settings section:
                     *   - add MapReduce framework name
                     */
                    if (!this.model.provider.entry.content.get('vix.mapreduce.framework.name')) {
                        this.model.provider.entry.content.set('vix.mapreduce.framework.name', 'yarn');
                    }

                    /** Kerberos Settings section:
                     *   - remove MapReduce Principal
                     *   - add Resourcemanager Principal and Nodemanager Principal
                     */
                    this.setKerberosMapReducePrincipla('');
                    this.children.mapreducePrincipal.$el.hide('fast');
                    this.children.resourcemanagerPrincipal.$el.show('fast');
                    this.children.nodemanagerPrincipal.$el.show('fast');
                }
                else {  // not choose 'yarn'
                    /** Hadoop Cluster Information section:
                     *   - show job tracker
                     *   - hide Resource Manager and Resource Scheduler and remove attributes
                     */
                    this.children.jobTracker.$el.show('fast');
                    this.model.provider.entry.content.set('vix.yarn.resourcemanager.address', '');
                    this.model.provider.entry.content.set('vix.yarn.resourcemanager.scheduler.address', '');
                    this.children.resourceManager.$el.hide();
                    this.children.resourceScheduler.$el.hide();

                    /** Additional Settings section:
                     *   - remove MapReduce framework name
                     */
                    this.model.provider.entry.content.set('vix.mapreduce.framework.name', '');

                    /** Kerberos Settings section:
                     *   - show MapReduce Principal
                     *   - hide Resourcemanager Principal and Nodemanager Principal
                     */
                    this.children.mapreducePrincipal.$el.show('fast');
                    this.model.provider.entry.content.set('vix.yarn.resourcemanager.principal', '');
                    this.model.provider.entry.content.set('vix.yarn.nodemanager.principal', '');
                    this.children.resourcemanagerPrincipal.$el.hide('fast');
                    this.children.nodemanagerPrincipal.$el.hide('fast');
                }
            },

            _onChangeFamily: function() {
                var family = this.model.provider.entry.content.get('vix.family');
                if (family !== 'hadoop') {
                    this.children.envVars.hide();
                    this.children.providerPaths.hide();
                    this.children.splunkSettings.hide();
                    $('#additional-settings p').hide(); // hide additional settings description for non-hadoop
                } else {
                    this.children.envVars.show();
                    this.children.providerPaths.show();
                    this.children.splunkSettings.show();
                    $('#additional-settings p').show();
                }
            },

            _onChooseKerberosConfigSwitch: function() {
                var flag = this.formModel.get(KERBEROS_CONFIG_SWITCH);

                if (flag === KERBEROS_CONFIG_SWITCH_SERVER_NAME_AND_REALM) {
                    // hide configuration file path field
                    this.setKerberosConfigFilePath('');
                    this.children.kerberosConfigurationFilePath.$el.hide('fast');

                    this.children.kerberosServerName.$el.show('fast');
                    this.children.kerberosDefaultRealm.$el.show('fast');
                }

                if (flag === KERBEROS_CONFIG_SWITCH_FILE_PATH) {
                    this.setKerberosServerName('');
                    this.setKerberosRealm('');
                    this.children.kerberosServerName.$el.hide('fast');
                    this.children.kerberosDefaultRealm.$el.hide('fast');

                    this.children.kerberosConfigurationFilePath.$el.show('fast');
                }
            },

            _syncOldKerberosAttributes: function() {
                var oldServerName = this.model.provider.entry.content.get(OLD_KERBEROS_SERVER_NAME);
                var oldRealm = this.model.provider.entry.content.get(OLD_KERBEROS_REALM);
                var newServerName = this.model.provider.entry.content.get(KERBEROS_SERVER_NAME);
                var newRealm = this.model.provider.entry.content.get(KERBEROS_REALM);

                newServerName = newServerName || oldServerName;
                newRealm = newRealm || oldRealm;

                this.setKerberosServerName(newServerName);
                this.setKerberosRealm(newRealm);
            },

            _toggleKerberosSectionAndMode: function() {
                // show/hide section and set/unset kerberos mode
                var modeSwitch = this.model.provider.entry.content.get(KERBEROS_AUTHORIZATION);
                if (modeSwitch) {
                    this.sections.kerberosSettings.show();
                    this.setKerberosMode(KERBEROS_MODE_DEFAULT);
                }
                else {
                    this.sections.kerberosSettings.hide();
                    this.setKerberosMode('');
                }
            },

            isYarn: function() {
                return this.model.provider.entry.content.get(HADOOP_VERSION) === HADOOP_2X_YARN;
            },

            setKerberosMode: function(value) {
                // by default, the mode should be 'kerberos'
                this.model.provider.entry.content.set(KERBEROS_MODE, value);
            },

            setKerberosServerName: function(value) {
                this.model.provider.entry.content.set(KERBEROS_SERVER_NAME, value);
            },

            setKerberosRealm: function(value) {
                this.model.provider.entry.content.set(KERBEROS_REALM, value);
            },

            setKerberosMapReducePrincipla: function(value) {
                this.model.provider.entry.content.set(KERBEROS_MAPREDUCE_PRINCIPAL, value);
            },

            setJobTracker: function(value) {
                this.model.provider.entry.content.set(JOB_TRACKER, value);
            },

            setKerberosConfigFilePath: function(value) {
                this.model.provider.entry.content.set(KERBEROS_CONFIGURATION_FILE_PATH, value);
            },

            render: function() {
                this.isEdit = (this.model.provider.get('mode') == 'edit');
                var html = this.compiledTemplate({
                    isEdit: this.isEdit,
                    managerRoute: this._managerRoute.bind(this)
                });

                var $html = $(html);
                if (!this.el.innerHTML) {
                    $html.find('.adminContent').append(this.children.flashMessages.render().el);

                    $html.find('.adminContent').append(this.children.basic.render().el);
                    $html.find('.adminContent').append(this.children.envVars.render().el);
                    $html.find('.adminContent').append(this.children.providerPaths.render().el);
                    $html.find('.adminContent').append(this.children.splunkSettings.render().el);
                    $html.find('.adminContent').append(this.children.addSecureCluster.render().el);
                    $html.find('.adminContent').append(this.children.kerberosSettings.render().el);
                    $html.find('.adminContent').append(this.children.additionalSettings.render().el);

                    $html.find('.adminContent').append('<div class="button-wrapper" />');
                    $html.find('.adminContent .button-wrapper').append('<a href="#" class="btn cancel pull-left">' + _('Cancel').t() + '</a>');
                    $html.find('.adminContent .button-wrapper').append('<a href="#" class="btn btn-primary pull-right">' + _('Save').t() + '</a>');
                }
                this.$el.html($html);

                // hide resource manager/scheduler in default
                if (!this.isYarn()) {
                    this.children.resourceManager.hide();
                    this.children.resourceScheduler.hide();
                }

                // hide kerberos server name, default realm and configuration file path in default,
                // only show it when user choose options in the drop-down list
                this.children.kerberosServerName.$el.hide();
                this.children.kerberosDefaultRealm.$el.hide();
                this.children.kerberosConfigurationFilePath.$el.hide();

                // hide kerberos settings in default case
                if (this.model.provider.entry.content.get(KERBEROS_AUTHORIZATION) !== '1') {
                    this.children.kerberosSettings.$el.hide();
                }

                // disable kerberos mode setting since we only support kerberos mode
                this.children.kerberosMode.childList[0].disable();

                // ERP-582: according to Allan's suggestion, add two sub-section labels
                if(this.children.kerberosConfig.$el.children('.subsection').length === 0) {
                    this.children.kerberosConfig.$el.prepend('<div class="subsection">' + _('Kerberos Server Configuration:').t() + '</div>');
                }
                if(this.children.kerberosPrincipalName.$el.children('.subsection').length === 0) {
                    this.children.kerberosPrincipalName.$el.prepend('<div class="subsection">' + _('Hadoop Kerberos Credentials:').t() + '</div>');
                }

                if (this.isEdit) {
                    this.children.name.childList[0].disable();
                    this.formLoadDfd.done(function() {
                        this.children.family.childList[0].disable();
                    }.bind(this));
                }
                return this;
            }
        });
    }
);
